

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <syslog.h>
#include <unistd.h>

/* For st */
#include <sys/ioctl.h>
#include <sys/mtio.h>

#include "ansiTape.h"
#include "ansiTapeLabel.h"

AnsiTape *openAnsiTape(const char *filename, const char *dev)
{
#if defined(MTIOCTOP)
	int tape;
	int fileSeqNum = 0;
	int stat;
	int i = 0;
	char vol[LABELSIZE];
	char label[LABELSIZE];

	AnsiTape *thisAnsiTape;
	struct mtop mtoperS, *mtoper = &mtoperS;

	if (NULL == (thisAnsiTape = malloc(sizeof(AnsiTape)))) {
		syslog(LOG_ERR, "Could not allocate memory for ansiTape.");
		return NULL;
	}
	tape = open(dev, O_RDWR);
	if (tape == -1) {
		syslog(LOG_ERR, "Could not open tape.");
		return NULL;
	}
#if defined(MTEOM)
	mtoper->mt_op = MTEOM;
#elif defined(MTSEOD)
	mtoper->mt_op = MTSEOD;
#else
#error Unknown ioctl operation
#endif
	mtoper->mt_count = 1;
	stat = ioctl(tape, MTIOCTOP, mtoper);
	if (stat == -1) {
		syslog(LOG_ERR, "Could not go to the eom of the tape.");
		return NULL;
	}
	mtoper->mt_op = MTBSF;
	/* BUGBUG this varies from tape to tape.     *
	 * Old DLT III drives seem to work with 2,   *
	 * the new DLT 8000 needs 3. To investigate. */
	mtoper->mt_count = 3;
	stat = ioctl(tape, MTIOCTOP, mtoper);
	if (stat == -1) {
		int isNewTape;

		mtoper->mt_op = MTREW;
		mtoper->mt_count = 1;
		stat = ioctl(tape, MTIOCTOP, mtoper);
		if (stat == -1) {
			syslog(LOG_ERR, "Could not rewind tape.");
			return NULL;
		}
		isNewTape = readVolumeLabel(tape, vol);

		stat = ioctl(tape, MTIOCTOP, mtoper);
		if (stat == -1) {
			syslog(LOG_ERR, "Could not rewind tape.");
			return NULL;
		}
		if (isNewTape == 0) {
			writeVolumeLabel(tape, vol);
		} else {
			writeNewVolumeLabel(tape, "HADAQ ", "              ");
		}

		fileSeqNum = 1;
	} else {
		mtoper->mt_op = MTFSF;
		mtoper->mt_count = 1;
		stat = ioctl(tape, MTIOCTOP, mtoper);
		if (stat == -1) {
			syslog(LOG_ERR, "Could not jump over the filemark.");
			return NULL;
		}
		do {
			stat = read(tape, label, LABELSIZE);
			if (i == 0) {
				sscanf(label, "%*31c%4d", &fileSeqNum);
				fileSeqNum++;
			}
			i++;
		} while (stat == LABELSIZE);
	}

	writeHeader(tape, fileSeqNum, filename);

	mtoper->mt_op = MTWEOF;
	mtoper->mt_count = 1;
	stat = ioctl(tape, MTIOCTOP, mtoper);
	if (stat == -1) {
		syslog(LOG_ERR, "Could not write filemark.");
		return NULL;
	}
	thisAnsiTape->fd = tape;

	thisAnsiTape->bytesWritten = 0;
	thisAnsiTape->bufferFull = 0;

	thisAnsiTape->filename = filename;
	thisAnsiTape->fileSeqNum = fileSeqNum;

	return thisAnsiTape;
#else
	return NULL;
#endif
}

int writeAnsiTape(AnsiTape *openTape, const char *src, size_t size)
{
#if defined(MTIOCTOP)
	int retVal = 0;
	while (size >= BLOCKSIZE - openTape->bufferFull) {
		if (openTape->bufferFull < BLOCKSIZE) {
			memcpy(openTape->buf + openTape->bufferFull, src, BLOCKSIZE - openTape->bufferFull);
			src += (BLOCKSIZE - openTape->bufferFull);
			retVal += (BLOCKSIZE - openTape->bufferFull);
			openTape->bytesWritten += (BLOCKSIZE - openTape->bufferFull);
			size -= (BLOCKSIZE - openTape->bufferFull);
		}
		write(openTape->fd, openTape->buf, BLOCKSIZE);
		openTape->bufferFull = 0;
	}
	if (size > 0) {
		memcpy(openTape->buf + openTape->bufferFull, src, size);
		openTape->bytesWritten += size;
		openTape->bufferFull += size;
		retVal += size;
	}
	return retVal;
#else
	return -1;
#endif
}

/* BUGBUG the last write may have less than BLOCKSIZE bytes */
int closeAnsiTape(AnsiTape *openTape)
{
#if defined(MTIOCTOP)
	int stat;
	struct mtop mtoperS, *mtoper = &mtoperS;
	if (openTape->bufferFull != 0) {
		write(openTape->fd, openTape->buf, openTape->bufferFull);
	}
	mtoper->mt_op = MTWEOF;
	mtoper->mt_count = 1;
	stat = ioctl(openTape->fd, MTIOCTOP, mtoper);
	if (stat == -1) {
		syslog(LOG_ERR, "Could not write filemark.");
		return -1;
	}
	writeTrailer(openTape->fd, openTape->fileSeqNum, openTape->bytesWritten, openTape->filename);

	stat = close(openTape->fd);
	if (stat == -1) {
		syslog(LOG_ERR, "Could not close file on tape.");
		return -1;
		free(openTape);
	}
	return 0;
#else
	return -1;
#endif
}
