
#define _XOPEN_SOURCE
#define _XOPEN_SOURCE_EXTENDED
#include <unistd.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <time.h>

/* For getpwent() */
#include <pwd.h>
#include <sys/types.h>
/* For getuid() */
#include <unistd.h>
#include <sys/types.h>

#include "ansiTape.h"
#include "ansiTapeChar.h"
#include "ansiTapeLabel.h"

/* Functions concerning the volume label */

int readVolumeLabel(int tape, char *vol)
{
	int stat;
	stat = read(tape, vol, LABELSIZE);
	if (stat != LABELSIZE) {
		return -1;
	}
	return 0;
}

int writeVolumeLabel(int tape, const char *vol)
{
	int stat;

	stat = write(tape, vol, LABELSIZE);
	if (stat != LABELSIZE) {
		syslog(LOG_ERR, "Could not write volume label to tape.");
		return -1;
	}
	return 0;
}

int writeNewVolumeLabel(int tape, const char *volId, const char *ownerId)
{
	int stat;
	char vol[LABELSIZE];
	char oid[15];
	const char *impId = "    HADAQ0023";

	strncpy(oid, ownerId, 15);
	if (LABELSIZE != sprintf(vol, "VOL1HADAQ               %s%s                            4", impId, ansichar(oid, 14))) {
		syslog(LOG_ERR, "Volume label not %d char.", LABELSIZE);
		return -1;
	}
	stat = write(tape, vol, LABELSIZE);
	if (stat != LABELSIZE) {
		syslog(LOG_ERR, "Could not write volume label to tape.");
		return -1;
	}
	return 0;
}

/* Functions concerning the hdr-label and the eof-label */

static int writeLabel(int tape, const char *hdreof, int fileSeqNum, int bytes, const char *filename)
{
	int stat;
	int i;
	int blockSize = BLOCKSIZE;
	char hostname[21];
	char *creaCent;
	char label[LABELSIZE + 1];
	char fn[37];
	const char *impId = "    HADAQ0023";
	uid_t uid;
	char *user;
	time_t tim;
	struct passwd pwentryS, *pwentry = &pwentryS;
	struct tm cdS, *cd = &cdS;

	tim = time(0);
	cd = gmtime(&tim);
	if ((cd->tm_year) / 100 == 19) {
		creaCent = " ";
	} else {
		creaCent = "0";
	}

	strcpy(fn, filename);
	if (LABELSIZE != sprintf(label, "%-3s%1d%-17s%06d%04d%04d%04d%02d%1s%02d%03d%6d%1s%06d%-13s%-7s",
							 hdreof,
							 1,
							 ansichar(fn, 17),
							 1,
							 1,
							 fileSeqNum,
							 1,
							 0,
							 creaCent, (cd->tm_year) % 100, cd->tm_yday,
							 99366,
							 " ",
							 (bytes + BLOCKSIZE - 1) / BLOCKSIZE,
							 impId,
							 "")
		) {
		syslog(LOG_ERR, "Label 1 not %d char.", LABELSIZE);
		return -1;
	}
	stat = write(tape, label, LABELSIZE);
	if (stat != LABELSIZE) {
		syslog(LOG_ERR, "Could not write ANSI label 1 on tape.");
		return -1;
	}
	if (LABELSIZE != sprintf(label, "%-3s%1d%1s%05d%05d%-21s%1s%010d%-3s%02d%28s",
							 hdreof,
							 2,
							 "F",
							 blockSize,
							 blockSize,
							 "",
							 "M",
							 bytes,
							 "",
							 0,
							 "")
		) {
		syslog(LOG_ERR, "Label 2 not %d char.", LABELSIZE);
		return -1;
	}
	stat = write(tape, label, LABELSIZE);
	if (stat != LABELSIZE) {
		syslog(LOG_ERR, "Could not write ANSI label 2 on tape.");
		return -1;
	}
	uid = getuid();
	while (NULL != (pwentry = getpwent()) && pwentry->pw_uid != uid) {
	}

	if (NULL != pwentry) {
		user = pwentry->pw_name;
	} else {
		user = "hades";
	}

	gethostname(hostname, 20);

	if (LABELSIZE != sprintf(label, "%-3s%1d%010d%-10s%-20s%-36s",
							 hdreof,
							 3,
							 tim,
							 user,
							 hostname,
							 filename)
		) {
		syslog(LOG_ERR, "Label 3 not %d char.", LABELSIZE);
		return -1;
	}
	stat = write(tape, label, LABELSIZE);
	if (stat != LABELSIZE) {
		syslog(LOG_ERR, "Could not write ANSI label 3 on tape.");
		return -1;
	}
	return 0;
}

int writeHeader(int tape, int fileSeqNum, const char *filename)
{
	return writeLabel(tape, "HDR", fileSeqNum, 0, filename);
}

int writeTrailer(int tape, int fileSeqNum, unsigned long numBytes, const char *filename)
{
	return writeLabel(tape, "EOF", fileSeqNum, numBytes, filename);
}
