proc createAgent {agent args} {
	global agents
	global agentList

	lappend agentList $agent
	set agents($agent.name) $agent
	set agents($agent.host) $agent
	set agents($agent.user) [eval lindex $args 0]
	set agents($agent.dir) [eval lindex $args 1]
	set agents($agent.status) down
}

proc deleteAgent {agent} {
	global agents
	global agentList

	unset agents($agent.name)
	unset agents($agent.host)
	unset agents($agent.user)
	unset agents($agent.dir)
	unset agents($agent.status)
	set agentList [lreplace $agentList [lsearch $agentList $agent] [lsearch $agentList $agent]]
}

proc catAgent {opts} {
	global agents
	global agentList

	set status ""
	if {[lsearch $opts "-up"] != -1} {
		set status up
	} elseif {[lsearch $opts "-down"] != -1} {
		set status down
	}
	set w ""
	if {[info exists agentList]} {
		foreach i $agentList {
			if {$status == "" || $status == $agents($i.status)} {
				lappend w $i
			}
		}
	}
	return $w
}

proc chkAgent {agent} {
	global agents

	if [catch {exec /usr/sbin/rpcinfo -t [agent list $agent host] 536870914}] {
		set agents($agent.status) down
	} else {
		set agents($agent.status) up
	}
}

proc connectAgent {agent} {
	global agents

    chkAgent $agent
    if {"$agents($agent.status)" == "up"} {
	set agents($agent.addr) [eval c_agent connect $agents($agent.host)]
    } else {
	#error "agent $agent not up"
	while {"$agents($agent.status)" != "up"} {chkAgent $agent}
	set agents($agent.addr) [eval c_agent connect $agents($agent.host)]
    }
	
}

proc disconnectAgent {agent} {
	global agents

	c_agent disconnect $agents($agent.addr)
	unset agents($agent.addr)
	set agents($agent.status) down
}

proc listAgent {agent what } {
	global agents

	if {$what == {}} {
		set what {name host user dir status}
	}
	foreach i $what {
		catch {unset m}
		if {[llength $what] != 1} {
			lappend m "$i"
		}
		lappend m "$agents($agent.$i)"
		lappend l "$m"
	}
	return $l
}

proc agent {oper {agent {}} {args {}}} {
	global agents

	if {"$oper" == "help"} {
		puts {usage: agent <oper> [options]}
	} elseif {"$oper" == "oper"} {
		puts "create delete connect disconnect status oper help"
	} elseif {"$oper" == "cat"} {
		catAgent "$agent $args"
	} elseif {"$oper" == "create"} {
		createAgent $agent $args
	} elseif {"$oper" == "connect"} {
		if {"$agent" == "all"} {
			while {[agent cat -down] != {}} {
				connectAgent [lindex [agent cat -down] 0]
			}
		} else {
			connectAgent $agent
		}
	} elseif {"$oper" == "disconnect"} {
		if {"$agent" == "all"} {
			while {[agent cat -up] != {}} {
				disconnectAgent [lindex [agent cat -up] [expr [llength [agent cat -up]] - 1]]
			}
		} else {
			disconnectAgent $agent
		}
	} elseif {"$oper" == "status"} {
		return $agents($agent.status)
	} elseif {"$oper" == "list"} {
		listAgent $agent $args
	} elseif {"$oper" == "delete"} {
		deleteAgent $agent
	}
}

proc createWorker {worker opts} {
	global workers
	global workerList

	lappend workerList $worker
	regexp {^([^:]*):([^:]*)$} $worker workers($worker.name) workers($worker.agent) workers($worker.file)
	set workers($worker.file) ./daq_$workers($worker.file)
	set workers($worker.opts) $opts
	set workers($worker.addOpts) {}
	set workers($worker.status) down
}

proc addOptsWorker {worker opts} {
	global workers

	set workers($worker.addOpts) $opts
}

proc deleteWorker {worker} {
	global workers
	global workerList

	unset workers($worker.name)
	unset workers($worker.agent)
	unset workers($worker.file)
	unset workers($worker.status)
	unset workers($worker.opts)
	unset workers($worker.addOpts)
	set workerList [lreplace $workerList [lsearch $workerList $worker] [lsearch $workerList $worker]]
}

proc catWorker {opts} {
	global workers
	global workerList

	set status ""
	if {[lsearch $opts "-up"] != -1} {
		set status up
	} elseif {[lsearch $opts "-down"] != -1} {
		set status down
	}
	set w ""
	if {[info exists workerList]} {
		foreach i $workerList {
			if {$status == "" || $status == $workers($i.status)} {
				lappend w $i
			}
		}
	}
	return $w
}

proc listWorker {worker what } {
	global workers

	if {$what == {}} {
		set what {name agent file opts status}
	}
	foreach i $what {
		catch {unset m}
		if {[llength $what] != 1} {
			lappend m "$i"
		}
		lappend m "$workers($worker.$i)"
		lappend l "$m"
	}
	return $l
}

proc startWorker {worker} {
	global workers

	set opts "$workers($worker.opts) $workers($worker.addOpts)"
	set workers($worker.addr) [eval c_worker start [agent list $workers($worker.agent) addr] $workers($worker.file) $opts]
	set workers($worker.status) up
}

proc stopWorker {worker} {
	global workers

	c_worker stop [agent list $workers($worker.agent) addr] $workers($worker.addr)
	unset workers($worker.addr)
	set workers($worker.status) down
}

proc worker {oper {worker {}} {args {}}} {
	global workers

	if {"$oper" == "help"} {
		puts {usage: worker <oper> [options]}
	} elseif {"$oper" == "oper"} {
		puts "create delete start stop status oper help"
	} elseif {"$oper" == "cat"} {
		catWorker "$worker $args"
	} elseif {"$oper" == "create"} {
		createWorker $worker $args
	} elseif {"$oper" == "addopts"} {
		addOptsWorker $worker $args
	} elseif {"$oper" == "start"} {
		if {"$worker" == "all"} {
			while {[worker cat -down] != {}} {
				startWorker [lindex [worker cat -down] 0]
			}
		} else {
			startWorker $worker
		}
	} elseif {"$oper" == "stop"} {
		if {"$worker" == "all"} {
			while {[worker cat -up] != {}} {
				stopWorker [lindex [worker cat -up] [expr [llength [worker cat -up]] - 1]]
			}
		} else {
			stopWorker $worker
		}
	} elseif {"$oper" == "status"} {
		if {[lsearch $args "-counter"] != -1 && $workers($worker.status) == "up"} {
			set l [c_worker status [agent list $workers($worker.agent) addr] $workers($worker.addr)]
		} else {
			set l $workers($worker.status)
		}
		return $l
	} elseif {"$oper" == "list"} {
		listWorker $worker $args
	} elseif {"$oper" == "delete"} {
		deleteWorker $worker
	}
}

proc createGroup {group} {
	global groups
	global groupList

	lappend groupList $group
	set groups($group.name) $group
}

proc deleteGroup {group} {
	global groups
	global groupList

	unset groups($group.name)
	set groupList [lreplace $groupList [lsearch $groupList $group] [lsearch $groupList $group]]
}

proc catGroup {opts} {
	global groups
	global groupList

	return $groupList
}

proc addGroup {group worker} {
	global groups

	lappend groups($group.members) $worker
}

proc removeGroup {group worker} {
	global groups

	set groups($group.members) [lreplace $groups($group.members) [lsearch $groups($group.members) $worker] [lsearch $groups($group.members) $worker]]
}

proc reverse {l} {
	set x [expr [llength $l] - 1]
	while {$x >= 0} {
		lappend m [lindex $l $x]
		set x [expr $x - 1]
	}
	return $m
}

proc group {oper {group {}} {args {}}} {
	global groups

	if {"$oper" == "oper"} {
		puts "create delete add remove oper help"
	} elseif {"$oper" == "cat"} {
		catGroup "$group $args"
	} elseif {"$oper" == "create"} {
		createGroup $group
	} elseif {"$oper" == "add"} {
		addGroup $group $args
	} elseif {"$oper" == "remove"} {
		removeGroup $group $args
	} elseif {"$oper" == "start"} {
		foreach i $groups($group.members) {
			worker start $i $args
		}
	} elseif {"$oper" == "stop"} {
		foreach i [reverse $groups($group.members)] {
			worker stop $i
		}
	} elseif {"$oper" == "status"} {
		catch {unset l}
		foreach i $groups($group.members) {
			lappend l [list $i [worker status $i $args]]
		}
		return $l
	} elseif {"$oper" == "list"} {
		foreach i {name members} {
			catch {unset m}
			lappend m "$i"
			lappend m "$groups($group.$i)"
			lappend l "$m"
		}
		return $l
	} elseif {"$oper" == "delete"} {
		deleteGroup $group
	} else {
		puts {usage: group <oper> [options]}
	}
}
