
static char *rcsId = "$Header: /misc/hadaq/cvsroot/hadaq/ctrlctu.c,v 6.17 2003/08/27 12:29:30 muench Exp $";

#define _XOPEN_SOURCE
#define _POSIX_C_SOURCE 199506L
#define SYSLOG_NAMES
#include <unistd.h>

#include <errno.h>
#include <stddef.h>
#include <setjmp.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <time.h>

#include "worker.h"

static jmp_buf terminateJmp;

void sigHandler(int sig)
{
	longjmp(terminateJmp, sig);
}

static void usage(const char *progName)
{
	syslog(LOG_ERR, "Usage: %s start|stop|#trig", progName);
}

int main(int argc, char *argv[])
{
	extern char *optarg;
	Worker *worker;
	int isStandalone;
	int priority;
	ptrdiff_t offset;
	int i;
	unsigned long value;
	int oper;
	unsigned long *ctuEnabled;
	struct timespec t1 = { 1, 0 };
	struct timespec t3 = { 3, 0 };

	openlog(argv[0], LOG_PID | LOG_PERROR, LOG_LOCAL0);
	setlogmask(LOG_UPTO(LOG_INFO));

	isStandalone = 1;
	priority = 0;
	while ((i = getopt(argc, argv, "ap:v:")) != -1) {
		switch (i) {
		case 'a':
			isStandalone = 0;
			break;
		case 'p':
			priority = atoi(optarg);
			break;
		case 'v':
			for (i = 0; prioritynames[i].c_name != NULL && 0 != strcmp(prioritynames[i].c_name, optarg); i++) {
			}
			if (prioritynames[i].c_name == NULL) {
				usage(argv[0]);
				goto bailOut0;
				exit(EXIT_FAILURE);
			} else {
				setlogmask(LOG_UPTO(prioritynames[i].c_val));
			}
			break;
		default:
			usage(argv[0]);
			goto bailOut0;
			break;
		}
	}

	if (NULL == (worker = Worker_initBegin(argv[0], sigHandler, priority, isStandalone))) {
		syslog(LOG_ERR, "connecting to agent: %s", strerror(errno));
		goto bailOut0;
	}
	if (NULL == (ctuEnabled = Worker_addStatistic(worker, "ctuEnabled"))) {
		syslog(LOG_ERR, "addingStatistic: %s", strerror(errno));
		goto bailOut0;
	}
	Worker_initEnd(worker);

	nanosleep(&t1, NULL);
	system("dtuctrl -t ctu reset");
	system("dtuctrl -t ctu start");
	(*ctuEnabled) = 1;
	if (0 == setjmp(terminateJmp)) {
		pause();
	}
	nanosleep(&t3, NULL);
	system("dtuctrl -t ctu stop");
	(*ctuEnabled) = 0;

	Worker_fini(worker);

	exit(0);

  bailOut0:
	exit(1);
}
