
#ifndef CTU_H
#define CTU_H

#include <assert.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <sys/types.h>
#include <ces/vmelib.h>

#include <stdint.h>


#define STATUS1		0x00
#define	DEAD1		0x04
#define CODE1		0x08
#define TAG1		0x0c
#define STATUS2		0x10
#define	DEAD2		0x14
#define CODE2		0x18
#define TAG2		0x1c
#define TRIG1		0x20
#define FIPO2		0x24
#define MEM_DATA1	0x30
#define MEM_DATA2	0x34
#define MEM_RESET	0x38
#define MAGIC		0x3c

typedef struct CtuS {
	volatile uint8_t *base;
	uint16_t internalTrigNr;
}

Ctu;

static uint8_t Ctu_rdStatus(Ctu * my)
{
	return *(my->base + STATUS1);
}

static uint8_t Ctu_rdDeadtime(Ctu * my)
{
	return *(my->base + DEAD1);
}

static void Ctu_wrDelay(Ctu * my, uint8_t delay)
{
	*(my->base + FIPO2) = delay;
}

static uint8_t Ctu_magicIsOk(Ctu * my)
{
	return *(my->base + MAGIC) == 0x51;
}

static void Ctu_clear(Ctu * my)
{
	*(my->base + STATUS1) |= 0x01;
	*(my->base + STATUS1) &= ~0x01;
	*(my->base + STATUS2) |= 0x01;
	*(my->base + STATUS2) &= ~0x01;
}

static void Ctu_inhibit(Ctu * my, int v)
{
	if (v) {
		*(my->base + STATUS1) |= 0x02;
		*(my->base + STATUS2) |= 0x02;
	} else {
		*(my->base + STATUS1) &= ~0x02;
		*(my->base + STATUS2) &= ~0x02;
	}
}

static void Ctu_maskBusy(Ctu * my, int v)
{
	if (v) {
		*(my->base + STATUS1) |= 0x40;
	} else {
		*(my->base + STATUS1) &= ~0x40;
	}
}

static void Ctu_maskTrig(Ctu * my, int v)
{
	if (v) {
		*(my->base + STATUS1) |= 0x08;
	} else {
		*(my->base + STATUS1) &= ~0x08;
	}
}

static void Ctu_lockBusy(Ctu * my, int v)
{
	if (v) {
		*(my->base + STATUS1) |= 0x01;
	} else {
		*(my->base + STATUS1) &= ~0x01;
	}
}

static void Ctu_softBusy(Ctu * my, int v)
{
	if (v) {
		*(my->base + STATUS1) |= 0x20;
	} else {
		*(my->base + STATUS1) &= ~0x20;
	}
}

static int Ctu_isBusy(Ctu * my)
{
	return (*(my->base + STATUS1) & 0x40) != 0;
}

static int Ctu_isError(Ctu * my)
{
	return (*(my->base + STATUS1) & 0x80) != 0;
}

static void Ctu_wrDeadtime(Ctu * my, uint8_t time)
{
	*(my->base + DEAD1) = time;
}

static uint8_t Ctu_rdTrigCode(Ctu * my)
{
	return *(my->base + CODE1) & 0x0f;
}

static uint8_t Ctu_rdTrigTag(Ctu * my)
{
	return *(my->base + TAG1);
}

static int Ctu_fifoIsEmpty(Ctu * my)
{
	return (*(my->base + FIPO2) & 0x01) != 0;
}

static void Ctu_popFifo(Ctu * my)
{
	*(my->base + FIPO2) = 0;
}

static void Ctu_softTrig(Ctu * my, uint8_t trigType)
{
	struct timespec tS, *t = &tS;
	t->tv_sec = 0;
	t->tv_nsec = 1;

	*(my->base + TRIG1) = 0x80;
	nanosleep(t, NULL);
	*(my->base + TRIG1) = 0x80 | trigType;
	nanosleep(t, NULL);
	*(my->base + TRIG1) = 0x00;
	nanosleep(t, NULL);
}

static char *Ctu_2charP(Ctu * my)
{
	static char s[7 * 80];

	sprintf(s,
			"base: 0x%08x\n"
			"status: 0x%02x\n"
			"deadtime: 0x%02x\n"
			"trigCode: 0x%01x\n"
			"trigTag: 0x%02x\n"
			"magicIsOk: %d\n",
			my->base,
			Ctu_rdStatus(my),
			Ctu_rdDeadtime(my),
			Ctu_rdTrigCode(my),
			Ctu_rdTrigTag(my),
			Ctu_magicIsOk(my)
		);
	return s;
}

static int Ctu_invariant(Ctu * my)
{
	return 1;
}

static int conCtu(Ctu * my, unsigned long cardBase)
{
	struct pdparam_master paramS, *param = &paramS;
	unsigned char init_code1[] =
	{0, 1, 4, 4, 5, 5, 5, 5,
	 6, 6, 6, 6, 6, 6, 6, 6,
	 2, 2, 2, 2, 2, 2, 2, 2,
	 2, 2, 2, 2, 2, 2, 2, 2,
	 3, 3, 3, 3, 3, 3, 3, 3,
	 3, 3, 3, 3, 3, 3, 3, 3,
	 3, 3, 3, 3, 3, 3, 3, 3,
	 3, 3, 3, 3, 3, 3, 3, 3};
	int i;

	assert(my != NULL);

	param->iack = 1;
	param->rdpref = 0;
	param->wrpost = 0;
	param->swap = 2;


	if (
		   (volatile uint8_t *) -1 == (my->base = (volatile uint8_t *) find_controller(
																			 cardBase, 0x10000UL, 0x09UL, 0x0000003cUL, 1UL, param
									   ))
		) {
		return -1;
	}
	*(my->base + STATUS1) = 0;
	*(my->base + STATUS2) = 0;
	for (i = 0; i < 64; ++i) {
		*(my->base + TAG1) = i;
		*(my->base + CODE1) = init_code1[i];
	}
	*(my->base + TAG1) = 0;
	assert(Ctu_invariant(my));

	return 0;
}

static void desCtu(Ctu * my)
{
	assert(Ctu_invariant(my));

	Ctu_inhibit(my, 0);
	return_controller((unsigned long) my->base, 0x10000UL);
}

#endif
