static char *rcsId = "$Header: /misc/hadaq/cvsroot/hadaq/evtbuild.c,v 6.74 2004/08/28 12:27:09 hadaq Exp $";


#define _POSIX_C_SOURCE 199309L
#define SYSLOG_NAMES

#include <unistd.h>

#include <errno.h>
#include <fcntl.h>
#include <limits.h>
#include <setjmp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/vfs.h>
#include <time.h>

#include <syslog.h>

#include <stdint.h>

#include <allParam.h>

#include "grmblfx.h"
#include "worker.h"
#include "evt.h"
#include "subevt.h"
#include "shmtrans.h"
#include "ansiTape.h"
#include "genid32.h"

#define NEVTIDS 64UL			/* must be 2^n */
#define NEVTIDS_IN_FILE 0UL		/* must be 2^n */

static FILE *outFile;
static AnsiTape *outTape;
static uint32_t seqNr;
static uint32_t runNr;
static time_t ourTime;

typedef struct TheArgsS {
	unsigned long nrOfMsgs;
	char outPath[PARAM_MAX_VALUE_LEN];
	char outDev[PARAM_MAX_VALUE_LEN];
	unsigned long runNr;
	char expId[PARAM_MAX_VALUE_LEN];
	char *slowCtrlFiles[PARAM_MAX_ARRAY_LEN];
	char slowCtrlFilesS[PARAM_MAX_ARRAY_LEN][PARAM_MAX_NAME_LEN];
	int slowCtrlFileCnt;
	unsigned long isStandalone;
	unsigned long priority;
	unsigned long queueSize;
	char verbosity[PARAM_MAX_VALUE_LEN];
	unsigned long evtId;
	off_t maxFileSz;
} TheArgs;

typedef struct TheStatsS {
	unsigned long *evtsDiscarded;
	unsigned long *evtsComplete;
	unsigned long *evtsDataError;
	unsigned long *evtsTagError;
	unsigned long *bytesWritten;
	unsigned long *evtId[NEVTIDS];
	unsigned long *trigNr[32];
} TheStats;

static jmp_buf terminateJmp;

void sigHandler(int sig)
{
	longjmp(terminateJmp, sig);
}

static void *appendFile(void *my, const char *path)
{
	void *subEvt;
	char *dataBuf;
	FILE *file;
	size_t fileSize;

	if (NULL == (file = fopen(path, "r"))) {
		syslog(LOG_ERR, "%s, %d: %s", __FILE__, __LINE__, strerror(errno));
		return my;
	}
	subEvt = newSubEvt(SubEvtDecoding_text, SubEvtId_slowTest, 0);

	if (NULL == (dataBuf = malloc(strlen(path + 2)))) {
		syslog(LOG_ERR, "%s, %d: %s", __FILE__, __LINE__, strerror(errno));
		return my;
	}
	sprintf(dataBuf, "#%s\n", path);
	subEvt = SubEvt_appendData(subEvt, dataBuf, strlen(dataBuf));
	free(dataBuf);

	fseek(file, 0, SEEK_END);
	fileSize = ftell(file);
	fseek(file, 0, SEEK_SET);
	if (NULL == (dataBuf = malloc(fileSize))) {
		syslog(LOG_ERR, "%s, %d: %s", __FILE__, __LINE__, strerror(errno));
		return my;
	}
	fread(dataBuf, 1, fileSize, file);

	subEvt = SubEvt_appendData(subEvt, dataBuf, fileSize);
	my = Evt_appendSubEvt(my, subEvt);

	deleteSubEvt(subEvt);
	free(dataBuf);
	fclose(file);

	return my;
}

static void usage(const char *progName)
{
	syslog(LOG_ERR, "Usage: %s [-x expId]", progName);
	syslog(LOG_ERR, "Usage: [-m nrOfMsgs] [-f slowCtrlFile ...]");
	syslog(LOG_ERR, "Usage: [-o outPath] [-d null|tape|file|stdout]");
	syslog(LOG_ERR, "Usage: [-q queueSize] [-r runNumber]");
	syslog(LOG_ERR, "Usage: [-a (agent)] [-p priority] [-I evtId]");
	syslog(LOG_ERR, "Usage: [-v debug|info|notice|warning|err|alert|crit|emerg]");
}

static void argsDump(TheArgs *my)
{
	int i;

	syslog(LOG_DEBUG, "nrOfMsgs: %d", my->nrOfMsgs);
	for (i = 0; i < my->slowCtrlFileCnt; i++) {
		syslog(LOG_DEBUG, "slowCtrlFiles[%d]: %s", i, my->slowCtrlFiles[i]);
	}
	syslog(LOG_DEBUG, "outPath: %s", my->outPath);
	syslog(LOG_DEBUG, "outDev: %s", my->outDev);
	syslog(LOG_DEBUG, "runNr: %d", my->runNr);
	syslog(LOG_DEBUG, "expId: %s", my->expId);
	syslog(LOG_DEBUG, "priority: %d", my->priority);
	syslog(LOG_DEBUG, "isStandalone: %d", my->isStandalone);
	syslog(LOG_DEBUG, "queueSize: %d", my->queueSize);
	syslog(LOG_DEBUG, "verbosity: %s", my->verbosity);
	syslog(LOG_DEBUG, "evtId: %ld", my->evtId);
	syslog(LOG_DEBUG, "maxFileSz: %ld", my->maxFileSz);
}

static void argsDefault(TheArgs *my)
{
	int i;

	my->nrOfMsgs = 0;
	for (i = 0; i < PARAM_MAX_ARRAY_LEN; i++) {
		my->slowCtrlFiles[i] = my->slowCtrlFilesS[i];
	}
	my->slowCtrlFileCnt = 0;
	strcpy(my->outPath, "");
	strcpy(my->outDev, "null");
	strcpy(my->expId, "xx");
	my->priority = 0;
	my->isStandalone = 1;
	my->queueSize = 1 * 1024 * 1024;
	strcpy(my->verbosity, "info");
	my->evtId = 0;
	my->maxFileSz = (2 * 1024 * 1024 * 1024UL - 1);
}

static int argsFromCL(TheArgs *my, int argc, char *argv[])
{
	extern char *optarg;
	int i;

	while ((i = getopt(argc, argv, "am:f:r:o:d:q:p:v:x:I:S:")) != -1) {
		switch (i) {
		case 'm':
			my->nrOfMsgs = strtoul(optarg, NULL, 0);
			break;
		case 'f':
			strcpy(my->slowCtrlFiles[my->slowCtrlFileCnt++], optarg);
			break;
		case 'o':
			strcpy(my->outPath, optarg);
			break;
		case 'd':
			strcpy(my->outDev, optarg);
			break;
		case 'x':
			strcpy(my->expId, optarg);
			break;
		case 'a':
			my->isStandalone = 0;
			break;
		case 'p':
			my->priority = strtoul(optarg, NULL, 0);
			break;
		case 'q':
			my->queueSize = strtoul(optarg, NULL, 0);
			break;
		case 'v':
			strcpy(my->verbosity, optarg);
			break;
		case 'I':
			my->evtId = strtoul(optarg, NULL, 0);
			break;
		case 'S':
			my->maxFileSz = strtoul(optarg, NULL, 0);
			break;
		default:
			return -1;
			break;
		}
	}
	return 0;
}

static int argsFromParam(TheArgs *my, int argc, char *argv[])
{
	Param paramS, *param = &paramS;
	int paramWasFound;
	char *name;

	conSetupParam(param, getenv("DAQ_SETUP"));

	name = (char *) basename(argv[0]);
	Param_getInt(param, name, "nrofmsgs", &paramWasFound, &my->nrOfMsgs);
	Param_getStringArray(param, name, "slwctrlfile", PARAM_MAX_ARRAY_LEN, &my->slowCtrlFileCnt, my->slowCtrlFiles);
	Param_getString(param, name, "outpath", &paramWasFound, my->outPath);
	Param_getString(param, name, "outdev", &paramWasFound, my->outDev);
	Param_getString(param, name, "expid", &paramWasFound, my->expId);
	Param_getInt(param, name, "stndln", &paramWasFound, &my->isStandalone);
	Param_getInt(param, name, "prio", &paramWasFound, &my->priority);
	Param_getInt(param, name, "qsize", &paramWasFound, &my->queueSize);
	Param_getString(param, name, "verb", &paramWasFound, my->verbosity);
	Param_getInt(param, name, "evtid", &paramWasFound, &my->evtId);
	Param_getInt(param, name, "maxfilesz", &paramWasFound, &my->maxFileSz);
	desParam(param);
}

static char *unit(unsigned long v)
{
	static char retVal[6];
	static char u[] = " kM";
	int i;

	for (i = 0; v >= 10000 && i < sizeof(u) - 2; v /= 1000, i++) {
	}
	sprintf(retVal, "%4d%c", v, u[i]);

	return retVal;
}

static void statsDump(TheArgs *theArgs, TheStats *my, int interval)
{
	static unsigned long lastEC;
	static unsigned long lastEE;
	static unsigned long lastTE;
	static unsigned long lastED;
	static unsigned long lastBW;
	static time_t t0 = 0;
	time_t t, dT;
	int i;


	if (theArgs->isStandalone && strcmp(theArgs->verbosity, "info") == 0) {
		t = time(NULL);
		dT = t - t0;
		if (dT >= interval) {
			int col = 0;

			fputs("==============================================================================\n", stderr);
			fprintf(stderr, "%19s:%6s", "evtsComplete  ", unit(*my->evtsComplete));
			fprintf(stderr, "%19s:%6s", "evtsDiscarded  ", unit(*my->evtsDiscarded));
			fprintf(stderr, "%19s:%6s", "bytesWritten  ", unit(*my->bytesWritten));
			fputc('\n', stderr);
			fprintf(stderr, "%19s:%6s", "evtsDataError  ", unit(*my->evtsDataError));
			fprintf(stderr, "%19s:%6s", "evtsTagError  ", unit(*my->evtsTagError));
			fputc('\n', stderr);

			fprintf(stderr, "%19s:%6s", "evtsComplete/s", unit((*my->evtsComplete - lastEC) / dT));
			fprintf(stderr, "%19s:%6s", "evtsDiscarded/s", unit((*my->evtsDiscarded - lastED) / dT));
			fprintf(stderr, "%19s:%6s", "bytesWritten/s", unit((*my->bytesWritten - lastBW) / dT));
			fputc('\n', stderr);
			fprintf(stderr, "%19s:%6s", "evtsDataError/s", unit((*my->evtsDataError - lastEE) / dT));
			fprintf(stderr, "%19s:%6s", "evtsTagError/s", unit((*my->evtsTagError - lastTE) / dT));
			fputc('\n', stderr);
			lastEC = *my->evtsComplete;
			lastEE = *my->evtsDataError;
			lastTE = *my->evtsTagError;
			lastED = *my->evtsDiscarded;
			lastBW = *my->bytesWritten;

			fputs("------------------------------------------------------------------------------\n", stderr);
			col = 0;
			for (i = 0; i < NEVTIDS; i++) {
				if (*my->evtId[i] != 0) {
					fprintf(stderr, "%17s%02x:%6s", "evtId", i, unit(*my->evtId[i]));
					if (++col == 3) {
						fputc('\n', stderr);
						col = 0;
					}
				}
			}
			if (col != 0) {
				fputc('\n', stderr);
			}
			col = 0;

			fputs("------------------------------------------------------------------------------\n", stderr);
			for (i = 0; i < theArgs->nrOfMsgs; i++) {
				fprintf(stderr, "%12s%02d: 0x%08x", "trigNr", i, *my->trigNr[i]);
				if (++col == 3) {
					fputc('\n', stderr);
					col = 0;
				}
			}
			if (col != 0) {
				fputc('\n', stderr);
			}
			t0 = t;
		}
	}
}


#define NJUNK 128
static void storeInfoStart(const char *n, time_t t, TheArgs *my)
{
	Param pS, *p = &pS;
	int i;
	char s[20];

	conSetupParam(p, getenv("DAQ_SETUP"));

	strftime(s, 20, "%Y-%m-%dT%H:%M:%S", localtime(&t));
	Param_storeString(p, n, "startdate", s);
	Param_storeInt(p, n, "nrofmsgs", my->nrOfMsgs);
	for (i = 0; i < my->slowCtrlFileCnt; i++) {
		sprintf(s, "slwctrlfile%d", i);
		Param_storeString(p, n, s, my->slowCtrlFiles[i]);
	}
	Param_storeString(p, n, "outpath", my->outPath);
	Param_storeString(p, n, "outdev", my->outDev);
	Param_storeInt(p, n, "runnr", my->runNr);
	Param_storeString(p, n, "expid", my->expId);
	Param_storeInt(p, n, "prio", my->priority);
	Param_storeInt(p, n, "stndln", my->isStandalone);
	Param_storeInt(p, n, "qsize", my->queueSize);
	Param_storeString(p, n, "verb", my->verbosity);
	/* This storing junk is for having the run start in
	 * oracle definately -- BS
	 */
	for (i = 0; i < NJUNK; i++) {
		Param_storeString(p, n, "junk", "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
	}
	desParam(p);
}

static void storeInfoStop(const char *n, time_t t, Worker *w)
{
	Param pS, *p = &pS;
	int i;
	char s[20];

	conSetupParam(p, getenv("DAQ_SETUP"));

	for (i = 0; i < 32 && strcmp(w->statistics[i].name, "") != 0; i++) {
		Param_storeInt(p, n, w->statistics[i].name, w->statistics[i].value);
	}
	strftime(s, 20, "%Y-%m-%dT%H:%M:%S", localtime(&t));
	Param_storeString(p, n, "stopdate", s);

	desParam(p);
}

static int openFile(TheArgs *theArgs)
{
	char fileName[_POSIX_PATH_MAX];
	static char outPath[_POSIX_PATH_MAX];
	static once = 1;

	runNr = genId32();
	seqNr = 0;

	theArgs->runNr = runNr;

	if (once) {
		strcpy(outPath, theArgs->outPath);
		once = 0;
	} else {
		strcpy(theArgs->outPath, outPath);
	}

	/* construct a default filename */
	strcpy(fileName, theArgs->expId);
	strftime(fileName + strlen(fileName), 18, "%y%j%H%M%S.hld", localtime(&ourTime));

	outTape = NULL;
	outFile = NULL;
	if (strcmp(theArgs->outDev, "null") == 0) {
		outFile = NULL;
	} else if (strcmp(theArgs->outDev, "stdout") == 0) {
		outFile = stdout;
	} else if (strcmp(theArgs->outDev, "file") == 0) {
		if (strcmp(theArgs->outPath, "") == 0) {
			strcpy(theArgs->outPath, fileName);
		} else {
			struct stat bufS, *buf = &bufS;

			stat(theArgs->outPath, buf);
			if (S_ISDIR(buf->st_mode)) {
				strcat(theArgs->outPath, "/");
				strcat(theArgs->outPath, fileName);
			}
		}

		if (NULL == (outFile = fopen(theArgs->outPath, "wb"))) {
			syslog(LOG_ERR, "opening file %s: %s", theArgs->outPath, strerror(errno));
			outFile = NULL;
			return -1;
		} else {
			struct statfs bufS, *buf = &bufS;
			statfs(theArgs->outPath, buf);
			if (theArgs->maxFileSz / buf->f_bsize > buf->f_bavail) {
				errno = ENOSPC;
				fclose(outFile);
				outFile = NULL;
				unlink(theArgs->outPath);
				syslog(LOG_ERR, "opening file %s: %s", theArgs->outPath, strerror(errno));
				return -1;
			}
		}
	} else if (strcmp(theArgs->outDev, "tape") == 0) {
		if (strcmp(theArgs->outPath, "") == 0) {
			strcpy(theArgs->outPath, fileName);
		}
		if (NULL == (outTape = openAnsiTape(theArgs->outPath, "/dev/tape"))) {
			syslog(LOG_ERR, "opening tape %s: %s", theArgs->outPath, strerror(errno));
			outFile = NULL;
			return -1;
		}
	} else {
		syslog(LOG_ERR, "unknown outputDev \"%s\"", theArgs->outDev);
		return -1;
	}
	return 0;
}

static int writeFile(void *evt)
{
	int writeFileR;

	Evt_setSeqNr(evt, seqNr++);
	Evt_setRunNr(evt, runNr);

	if (outFile != NULL) {
		writeFileR = fwrite(evt, 1, Evt_paddedSize(evt), outFile);
	} else if (outTape != NULL) {
		writeFileR = writeAnsiTape(outTape, evt, Evt_paddedSize(evt));
	}
	return writeFileR;
}

static int closeFile()
{
	int closeFileR;

	if (outFile != NULL) {
		closeFileR = fclose(outFile);
	} else if (outTape != NULL) {
		closeFileR = closeAnsiTape(outTape);
	}
	return closeFileR;
}

/* BUGBUG bailOut not proper yet */
int main(int argc, char *argv[])
{
	int i;
	TheArgs theArgsS, *theArgs = &theArgsS;
	TheStats theStatsS, *theStats = &theStatsS;
	ShmTrans **shmTrans;
	HadTuQueue **hadTuQueue;
	Worker *worker;

	int scanWasSuccessful;
	uint32_t currTrigNr;
	uint32_t currTrigTag;
	uint32_t currId;

	openlog(argv[0], LOG_PID | LOG_PERROR, LOG_LOCAL0);
	setlogmask(LOG_UPTO(LOG_INFO));

	argsDefault(theArgs);
	argsFromParam(theArgs, argc, argv);
	if (0 > argsFromCL(theArgs, argc, argv)) {
		usage(argv[0]);
		exit(EXIT_FAILURE);
	}
	for (i = 0; prioritynames[i].c_name != NULL && 0 != strcmp(prioritynames[i].c_name, theArgs->verbosity); i++) {
	}
	if (prioritynames[i].c_name == NULL) {
		exit(EXIT_FAILURE);
	} else {
		setlogmask(LOG_UPTO(prioritynames[i].c_val));
	}

	/* normalize experiment id */
	theArgs->expId[0] = tolower(theArgs->expId[0]);
	theArgs->expId[1] = tolower(theArgs->expId[1]);
	theArgs->expId[2] = '\0';

	if (NULL == (worker = Worker_initBegin(argv[0], sigHandler, theArgs->priority, theArgs->isStandalone))) {
		syslog(LOG_ERR, "%s, %d: %s", __FILE__, __LINE__, strerror(errno));
		exit(EXIT_FAILURE);
	}
	if (-1 == initOnline()) {
		syslog(LOG_WARNING, "unable to initialize online service");
	}
	if (theArgs->nrOfMsgs == 0) {
		/*
		   no '-m' option was on command line, we assume that the
		   readout task (daq_readout) is running on the same node and
		   communicates directly via shared memory
		 */
		syslog(LOG_NOTICE, "nrOfMsgs not set, assuming readout is local");

		shmTrans = malloc(sizeof(ShmTrans *));
		hadTuQueue = malloc(sizeof(HadTuQueue *));

		shmTrans[0] = ShmTrans_create("subevtqueue", 2 * theArgs->queueSize);
		hadTuQueue[0] = NULL;
		theArgs->nrOfMsgs = 1;
	} else {
		shmTrans = malloc(theArgs->nrOfMsgs * sizeof(ShmTrans *));
		hadTuQueue = malloc(theArgs->nrOfMsgs * sizeof(HadTuQueue *));

		for (i = 0; i < theArgs->nrOfMsgs; i++) {
			char buf[_POSIX_PATH_MAX];

			sprintf(buf, "netqueue%d", i);
			shmTrans[i] = ShmTrans_create(buf, 2 * theArgs->queueSize);
			hadTuQueue[i] = NULL;
		}
	}

	theStats->evtsDiscarded = Worker_addStatistic(worker, "evtsDiscarded");
	theStats->evtsComplete = Worker_addStatistic(worker, "evtsComplete");
	theStats->evtsDataError = Worker_addStatistic(worker, "evtsDataError");
	theStats->evtsTagError = Worker_addStatistic(worker, "evtsTagError");
	theStats->bytesWritten = Worker_addStatistic(worker, "bytesWritten");
	for (i = 0; i < NEVTIDS; i++) {
		char buf[WORKER_MAX_NAME_LEN];

		sprintf(buf, "evtId%d", i);
		theStats->evtId[i] = Worker_addStatistic(worker, buf);
	}
	for (i = 0; i < theArgs->nrOfMsgs; i++) {
		char buf[WORKER_MAX_NAME_LEN];

		sprintf(buf, "trigNr%d", i);
		theStats->trigNr[i] = Worker_addStatistic(worker, buf);
	}

	argsDump(theArgs);

	Worker_initEnd(worker);

	currId = 0;
	while (setjmp(terminateJmp) == 0) {
		void *evt;
		void *subEvt;
		int step;
		int evtIsBroken = 0;
		int dataError = 0;
		int tagError = 0;

		statsDump(theArgs, theStats, 1);

		if (*theStats->bytesWritten == 0) {
			ourTime = time(NULL);
			if (-1 == openFile(theArgs)) {
				syslog(LOG_ERR, "error opening output file, exiting");
				exit(EXIT_FAILURE);
			}
			storeInfoStart(argv[0], ourTime, theArgs);

			evt = newEvt(EvtDecoding_64bitAligned, EvtId_runStart);
			for (i = 0; i < theArgs->slowCtrlFileCnt; i++) {
				evt = appendFile(evt, theArgs->slowCtrlFiles[i]);
			}
			(*theStats->bytesWritten) += Evt_size(evt);
			writeFile(evt);
			deleteEvt(evt);
		}
		evt = newEvt(EvtDecoding_64bitAligned, EvtId_data);
		for (i = 0; i < theArgs->nrOfMsgs && !evtIsBroken; i += step) {
			uint32_t trigNr;
			uint32_t trigTag;

			if (hadTuQueue[i] == NULL) {
				void *storage;

				storage = ShmTrans_recv(shmTrans[i]);
				hadTuQueue[i] = malloc(HadTuQueue_sizeOf());
				conHadTuQueue_voidP(hadTuQueue[i], storage);
			}
			subEvt = HadTuQueue_front(hadTuQueue[i]);
#ifndef NDEBUG
			syslog(LOG_DEBUG, "hadTuQueue[%d]: %p = subEvt: %s", i, subEvt, SubEvt_2charP(subEvt));
#endif

			(*theStats->trigNr[i]) = SubEvt_trigNr(subEvt);

			if (i == 0) {
				currTrigNr = SubEvt_trigNr(subEvt) >> 8;
				currTrigTag = SubEvt_trigNr(subEvt) & 0xff;
				if (theArgs->evtId != 0) {
					currId = theArgs->evtId;
				} else {
					currId = SubEvt_pureId(subEvt);
				}
				syslog(LOG_DEBUG, "currTrigNr: 0x%06x, currTrigTag 0x%02x, currId 0x%08x", currTrigNr, currTrigTag, currId);
			}
			trigNr = SubEvt_trigNr(subEvt) >> 8;
			trigTag = SubEvt_trigNr(subEvt) & 0xff;
			if (trigNr == currTrigNr) {
				if (SubEvt_size(subEvt) > SubEvt_hdrSize()) {
					/* sub evt is not empty */
					if (SubEvt_dataError(subEvt)) {
						dataError = 1;
					}
					if (trigTag != currTrigTag) {
						tagError = 1;
					}
					evt = Evt_appendSubEvt(evt, subEvt);
				}
				HadTuQueue_pop(hadTuQueue[i]);
				step = 1;
			} else if (trigNr < currTrigNr) {
				/* BUGBUG subevt discarded, not in statistic */
				HadTuQueue_pop(hadTuQueue[i]);
				step = 0;
			} else {
				evtIsBroken = 1;
			}
			if (HadTuQueue_empty(hadTuQueue[i])) {
				desHadTuQueue(hadTuQueue[i]);
				free(hadTuQueue[i]);
				hadTuQueue[i] = NULL;
				ShmTrans_free(shmTrans[i]);
			}
		}
		if (!evtIsBroken) {
			(*theStats->evtsComplete)++;

			(*theStats->evtId[currId & (NEVTIDS - 1)])++;
			Evt_setId(evt, currId & (NEVTIDS_IN_FILE - 1));

			if (dataError) {
				Evt_setDataError(evt);
				(*theStats->evtsDataError)++;
			}
			if (tagError) {
				Evt_setDataError(evt);
				(*theStats->evtsTagError)++;
			}
			(*theStats->bytesWritten) += Evt_size(evt);
			writeFile(evt);
			Evt_online(evt);
		} else {
			(*theStats->evtsDiscarded)++;
		}
		deleteEvt(evt);
		if (*theStats->bytesWritten >= theArgs->maxFileSz - theArgs->queueSize) {
			evt = newEvt(EvtDecoding_64bitAligned, EvtId_runStop);
			for (i = 0; i < theArgs->slowCtrlFileCnt; i++) {
				evt = appendFile(evt, theArgs->slowCtrlFiles[i]);
			}
			(*theStats->bytesWritten) += Evt_size(evt);
			writeFile(evt);
			deleteEvt(evt);

			ourTime = time(NULL);
			closeFile();
			storeInfoStop(argv[0], ourTime - 2, worker);

			(*theStats->bytesWritten) = 0;
			(*theStats->evtsComplete) = 0;
			(*theStats->evtsDiscarded) = 0;
			(*theStats->evtsDataError) = 0;
			(*theStats->evtsTagError) = 0;
			for (i = 0; i < theArgs->nrOfMsgs; i++) {
				(*theStats->trigNr[i]) = 0;
			}
			for (i = 0; i < NEVTIDS; i++) {
				(*theStats->evtId[i]) = 0;
			}
		}
	}

	ourTime = time(NULL);
	closeFile();
	storeInfoStop(argv[0], ourTime - 2, worker);

	statsDump(theArgs, theStats, 1);

	for (i = 0; i < theArgs->nrOfMsgs; i++) {
		ShmTrans_remove(shmTrans[i]);
	}
	finiOnline();
	Worker_fini(worker);

	exit(EXIT_SUCCESS);
}
