#!./daq_xmanage

source cmds.tcl
source [lindex $argv 0]

set runInfo(name) {}
set runInfo(user) {}
set runInfo(outputType) null
set runInfo(output) {}
set runInfo(comment) {}

proc writeFile {name cont} {
	set f [open $name w]
	foreach line $cont {
		puts $f $line
	}
	close $f
}

proc updateViewFile {top name} {
	.$top.txt configure -state normal
	.$top.txt delete 1.0 end
	set f [open $name r]
	while {[gets $f line] >= 0} {
		.$top.txt insert end "$line\n"
	}
	close $f
	.$top.txt see end
	.$top.txt configure -state disabled
}

proc pollViewFile {top name} {
	global mtime

	if {[winfo exists .$top]} {
		if {[file mtime $name] > $mtime($name)} {
			updateViewFile $top $name
		}
		set mtime($name) [file mtime $name]
		after 1000 pollViewFile $top $name
	}
}

proc viewFile {top title name} {
	global mtime

	toplevel .$top
	wm title .$top "$title"
	wm iconname .$top "$title"

	text .$top.txt -state disabled -yscrollcommand ".$top.scroll set"
	scrollbar .$top.scroll -command ".$top.txt yview"
	pack .$top.txt -side left -fill both -expand 1
	pack .$top.scroll -side right -fill y

	set mtime($name) 0
	pollViewFile $top $name
}

proc writeRunInfo {} {
  global runInfo

	lappend l [list set runInfo(name) $runInfo(name)]
	lappend l [list set runInfo(user) $runInfo(user)]
	lappend l [list set runInfo(output) $runInfo(output)]
	lappend l [list set runInfo(comment) $runInfo(comment)]
	writeFile runinfo.tcl $l
}

proc distrFile {name} {
	set f [open $name r]
	while {[gets $f line] >= 0} {
		lappend l "$line"
	}
	foreach agent [agent cat] {
#		dp_RPC [agent list $agent addr] writeFile $name $l
	}
}

proc updateRunButtons {} {
	if [isDown [group status run]] {
		.mbar.file.menu entryconfigure 0 -state normal
		.mbar.file.menu entryconfigure 1 -state disabled
	} else {
		.mbar.file.menu entryconfigure 0 -state disabled
		.mbar.file.menu entryconfigure 1 -state normal
	}
}

proc outputTypeCmd {} {
	global runInfo

	if {"$runInfo(outputType)" == "null"} {
		.startRunDlg.top.output.ent configure -state normal
		set runInfo(output) /dev/null
		.startRunDlg.top.output.ent delete 0 end
		.startRunDlg.top.output.ent insert 0 $runInfo(output)
		.startRunDlg.top.output.ent configure -state disabled
	} elseif {"$runInfo(outputType)" == "tape"} {
		.startRunDlg.top.output.ent configure -state normal
    set runInfo(output) /dev/nrmt0h
		.startRunDlg.top.output.ent delete 0 end
		.startRunDlg.top.output.ent insert 0 $runInfo(output)
		.startRunDlg.top.output.ent configure -state disabled
  } elseif {"$runInfo(outputType)" == "file"} {
		.startRunDlg.top.output.ent configure -state normal
    set runInfo(output) {}
		.startRunDlg.top.output.ent delete 0 end
		.startRunDlg.top.output.ent insert 0 $runInfo(output)
  }
}

proc startRunDlg {} {
	global runInfo
	global dlgButton

	toplevel .startRunDlg
	wm title .startRunDlg "Start Run"
	wm iconname .startRunDlg "Start Run"
	frame .startRunDlg.top -relief raised -bd 1
	pack .startRunDlg.top -side top -fill both
	frame .startRunDlg.bottom -relief raised -bd 1
	pack .startRunDlg.bottom -side bottom -fill both

	frame .startRunDlg.top.name
	pack .startRunDlg.top.name -side top
	label .startRunDlg.top.name.lbl -text "Run Name:" -width 20 -anchor ne
	entry .startRunDlg.top.name.ent -width 20
	.startRunDlg.top.name.ent insert 0 $runInfo(name)
	pack .startRunDlg.top.name.lbl .startRunDlg.top.name.ent -side left

	frame .startRunDlg.top.user
	pack .startRunDlg.top.user -side top
	label .startRunDlg.top.user.lbl -text "Started by:" -width 20 -anchor ne
	entry .startRunDlg.top.user.ent -width 20
	.startRunDlg.top.user.ent insert 0 $runInfo(user)
	pack .startRunDlg.top.user.lbl .startRunDlg.top.user.ent -side left

	frame .startRunDlg.top.output
	pack .startRunDlg.top.output -side top
	frame .startRunDlg.top.output.frm
	label .startRunDlg.top.output.frm.lbl -text "Output to:" -width 20 -anchor ne
	frame .startRunDlg.top.output.frm.b
	entry .startRunDlg.top.output.ent -width 20
	outputTypeCmd
	pack .startRunDlg.top.output.frm .startRunDlg.top.output.ent -side left
	pack .startRunDlg.top.output.frm.lbl .startRunDlg.top.output.frm.b -side left
	radiobutton .startRunDlg.top.output.frm.b.null -text Null -variable runInfo(outputType) -value null -command outputTypeCmd
	radiobutton .startRunDlg.top.output.frm.b.tape -text Tape -variable runInfo(outputType) -value tape -command outputTypeCmd
	radiobutton .startRunDlg.top.output.frm.b.file -text File -variable runInfo(outputType) -value file -command outputTypeCmd
	pack .startRunDlg.top.output.frm.b.null .startRunDlg.top.output.frm.b.tape .startRunDlg.top.output.frm.b.file -side top

	frame .startRunDlg.top.comment
	pack .startRunDlg.top.comment -side top
	label .startRunDlg.top.comment.lbl -text "Comment:" -width 20 -anchor ne
	text .startRunDlg.top.comment.ent -width 20 -height 5
	.startRunDlg.top.comment.ent insert 1.0 "$runInfo(comment)"
	pack .startRunDlg.top.comment.lbl .startRunDlg.top.comment.ent -side left

	set dlgButton "cancel"
	button .startRunDlg.bottom.ok -text "OK" -command {set dlgButton "ok"}
	pack .startRunDlg.bottom.ok -side left
	button .startRunDlg.bottom.cancel -text "Cancel" -command {set dlgButton "cancel"}
	pack .startRunDlg.bottom.cancel -side right

	grab set .startRunDlg
	tkwait variable dlgButton
	if {"$dlgButton" == "ok"} {
		set runInfo(name) [.startRunDlg.top.name.ent get]
		set runInfo(user) [.startRunDlg.top.user.ent get]
		set runInfo(output) [.startRunDlg.top.output.ent get]
		set runInfo(comment) [.startRunDlg.top.comment.ent get 1.0 end]
	}
	destroy .startRunDlg
	return $dlgButton
}

proc stopRunDlg {} {
	global runInfo
	global dlgButton

	toplevel .stopRunDlg
	wm title .stopRunDlg "Stop Run"
	wm iconname .stopRunDlg "Stop Run"
	frame .stopRunDlg.top -relief raised -bd 1
	pack .stopRunDlg.top -side top -fill both
	frame .stopRunDlg.bottom -relief raised -bd 1
	pack .stopRunDlg.bottom -side bottom -fill both

	frame .stopRunDlg.top.user
	pack .stopRunDlg.top.user -side top
	label .stopRunDlg.top.user.lbl -text "Stopped by:" -width 20 -anchor ne
	entry .stopRunDlg.top.user.ent -width 20
	.stopRunDlg.top.user.ent insert 0 $runInfo(user)
	pack .stopRunDlg.top.user.lbl .stopRunDlg.top.user.ent -side left

	frame .stopRunDlg.top.comment
	pack .stopRunDlg.top.comment -side top
	label .stopRunDlg.top.comment.lbl -text "Comment:" -width 20 -anchor ne
	text .stopRunDlg.top.comment.ent -width 20 -height 5
	.stopRunDlg.top.comment.ent insert 1.0 "$runInfo(comment)"
	pack .stopRunDlg.top.comment.lbl .stopRunDlg.top.comment.ent -side left

	set dlgButton "cancel"
	button .stopRunDlg.bottom.ok -text "OK" -command {set dlgButton "ok"}
	pack .stopRunDlg.bottom.ok -side left
	button .stopRunDlg.bottom.cancel -text "Cancel" -command {set dlgButton "cancel"}
	pack .stopRunDlg.bottom.cancel -side right

	grab set .stopRunDlg
	tkwait variable dlgButton
	if {"$dlgButton" == "ok"} {
		set runInfo(user) [.stopRunDlg.top.user.ent get]
		set runInfo(comment) [.stopRunDlg.top.comment.ent get 1.0 end]
	}
	destroy .stopRunDlg
	return $dlgButton
}

proc startRunCmd {} {
	global runInfo
	global tapeInfo

  foreach worker [worker cat] {
		if {[string first evtbuild $worker] != -1} {
			set evtbuildWorker $worker
		}
	}

	if {[startRunDlg] == "ok"} {
		writeRunInfo
#		distrFile slowctrl.tcl
		worker addopts $evtbuildWorker -o $runInfo(output) -d $runInfo(outputType)
		group start run
	}
	updateRunButtons
}

proc stopRunCmd {} {
	global runInfo
	global tapeInfo

	if {[stopRunDlg] == "ok"} {
		group stop run
	}
	updateRunButtons
}

proc viewStat {top title statList} {
	toplevel .$top
	wm title .$top "$title"
	wm iconname .$top "$title"

	foreach worker $statList {
		set name [lindex $worker 0]

		frame .$top.$name
		pack .$top.$name -side top
		label .$top.$name.lbl -text $name
		text .$top.$name.txt -width 40 -height 7
		pack .$top.$name.lbl .$top.$name.txt -side top
	}
}

proc updateStat {top countList} {
	foreach worker $countList {
		set name [lindex $worker 0]
		set pairs [lindex $worker 1]

		.$top.$name.txt configure -state normal
		.$top.$name.txt delete 1.0 end
		foreach pair $pairs {
			set p0 [lindex $pair 0]
			set p1 [lindex $pair 1]
			
			if {$p1 != "down"} {
				.$top.$name.txt insert end [format "%19s:%14d\n" $p0 $p1]
			}
		}
		.$top.$name.txt configure -state disabled
	}
}

proc isDown {statList} {
	foreach pair $statList {
		set p0 [lindex $pair 0]
		set p1 [lindex $pair 1]
		
		if {$p1 == "down"} {
			return 1
		}
	}
	return 0
}

proc updateRunStat {} {
	if {![isDown [group status run]]} {
		updateStat viewRunStat [group status run -counter]
	}
}

proc pollRunStat {} {
	if {[catch updateRunStat] == 0} {
		after 5000 pollRunStat
	}
}

proc viewRunCmd {} {
	viewStat viewRunStat "Run Status" [group status run]
	pollRunStat
}

proc updateAcquisitionButtons {} {
	if [isDown [group status acquisition]] {
		.mbar.options.menu entryconfigure 0 -state normal
		.mbar.options.menu entryconfigure 1 -state disabled
		.mbar.options.menu entryconfigure 2 -state disabled
	} else {
		.mbar.options.menu entryconfigure 0 -state disabled
		.mbar.options.menu entryconfigure 1 -state normal
		.mbar.options.menu entryconfigure 2 -state normal
	}
}

proc execOnAgent {cmd} {
	set agent [lindex $cmd 0]
	set command [lindex $cmd 1]
	puts "agent $agent: command $command"
	if {$agent == "join"} {
		set b [c_joinall]
	} else {
		set b [c_system $agent $command]
	}
}

proc startAcquisitionCmd {} {
	global beforeStartAcq afterStartAcq
	foreach cmd $beforeStartAcq {
		execOnAgent $cmd
	}
	group start acquisition
	foreach cmd $afterStartAcq {
		execOnAgent $cmd
	}
	updateAcquisitionButtons
}

proc stopAcquisitionCmd {} {
	global beforeStopAcq afterStopAcq
	foreach cmd $beforeStopAcq {
		execOnAgent $cmd
	}
	group stop acquisition
	foreach cmd $afterStopAcq {
		execOnAgent $cmd
	}
	updateAcquisitionButtons
}

proc restartAcquisitionCmd {} {
	stopAcquisitionCmd
	startAcquisitionCmd
}

proc updateAcquisitionStat {} {
	if {![isDown [group status acquisition]]} {
		updateStat viewAcquisitionStat [group status acquisition -counter]
	}
}

proc pollAcquisitionStat {} {
	if {[catch updateAcquisitionStat] == 0} {
		after 5000 pollAcquisitionStat
	}
}

proc viewAcquisitionCmd {} {
	viewStat viewAcquisitionStat "Acquisition Status" [group status acquisition]
	pollAcquisitionStat
}

proc viewErrorlogCmd {} {
	viewFile viewErrorLog "Error Log" hadaq.log
}

proc viewEventCmd {} {

  foreach worker [worker cat] {
		if {[string first evtbuild $worker] != -1} {
			set evtbuildAgent [worker list $worker agent]
		}
	}
	set f [open daq_anal.out w]
	puts $f "Waiting for event"
	close $f
	after 1000
	viewFile viewEvt "View Event" daq_anal.out
	exec sh -c "./daq_sniff -h [agent list $evtbuildAgent host] | ./daq_anal -n 1" </dev/null >daq_anal.out 2>/dev/null &
}

if {$tk_version < 4.0} {
	set tearoff {}
} else {
	set tearoff {-tearoff 0}
}

proc loadTapeDlg {} {
	global runInfo
	global dlgButton

	toplevel .loadTapeDlg
	wm title .loadTapeDlg "Load Tape"
	wm iconname .loadTapeDlg "Load Tape"
	frame .loadTapeDlg.top -relief raised -bd 1
	pack .loadTapeDlg.top -side top -fill both
	frame .loadTapeDlg.bottom -relief raised -bd 1
	pack .loadTapeDlg.bottom -side bottom -fill both

	label .loadTapeDlg.top.lbl -text "Insert Tape"

	checkbutton .loadTapeDlg.top.overWr -text "Initialize Tape" -variable overwriteTape
	pack .loadTapeDlg.top.lbl .loadTapeDlg.top.overWr -side top

	set dlgButton "cancel"
	button .loadTapeDlg.bottom.ok -text "OK" -command {set dlgButton "ok"}
	pack .loadTapeDlg.bottom.ok -side left
	button .loadTapeDlg.bottom.cancel -text "Cancel" -command {set dlgButton "cancel"}
	pack .loadTapeDlg.bottom.cancel -side right
	grab set .loadTapeDlg
	tkwait variable dlgButton
	destroy .loadTapeDlg
	return $dlgButton
}

proc loadTapeCmd {} {
	global overwriteTape
	global tapeInfo

	set overwriteTape 0
	if {[loadTapeDlg] == "ok"} {
		set c [lindex [. config -cursor] 4]
		. config -cursor clock
		update
		while 1 {
			if {[catch {exec mt online}] == 0} {
				break
			}
			after 1000
		}
		if {$overwriteTape} {
			set tapeInfo(vol) "VOL1ULTRIX                           HADES                                     3"
			exec mt rewind
		} else {
			set tapeInfo(vol) {}
			exec mt seod
			exec mt bsf 1
		}
		. config -cursor $c
		update
	}
}

proc ejectTapeCmd {} {
	set c [lindex [. config -cursor] 4]
	. config -cursor clock
	update
	exec mt offline
	. config -cursor $c
	update
}

wm title . hadaq
frame .mbar -relief raised -bd 1

menubutton .mbar.file -text File -menu .mbar.file.menu
menubutton .mbar.view -text View -menu .mbar.view.menu
menubutton .mbar.options -text Options -menu .mbar.options.menu
menubutton .mbar.help -text Help -menu .mbar.help.menu
pack .mbar.file .mbar.view .mbar.options -side left
pack .mbar.help -side right

eval menu .mbar.file.menu $tearoff
.mbar.file.menu add command -label "Start Run" -command startRunCmd
.mbar.file.menu add command -label "Stop Run" -command stopRunCmd
.mbar.file.menu add separator
.mbar.file.menu add command -label "Load Tape" -command loadTapeCmd
.mbar.file.menu add command -label "Eject Tape" -command ejectTapeCmd
.mbar.file.menu add separator
.mbar.file.menu add command -label Exit -command {exit}

eval menu .mbar.view.menu $tearoff
.mbar.view.menu add command -label "Acquisition" -command viewAcquisitionCmd
.mbar.view.menu add command -label "Run" -command viewRunCmd
.mbar.view.menu add command -label "Tape" -command viewTapeCmd -state disabled
.mbar.view.menu add separator
.mbar.view.menu add command -label "Errorlog" -command viewErrorlogCmd
.mbar.view.menu add separator
.mbar.view.menu add command -label "Event" -command viewEventCmd

eval menu .mbar.options.menu $tearoff
.mbar.options.menu add command -label "Start Acquisition" -command startAcquisitionCmd
.mbar.options.menu add command -label "Stop Acquisition" -command stopAcquisitionCmd
.mbar.options.menu add command -label "Restart Acquisition" -command restartAcquisitionCmd

eval menu .mbar.help.menu $tearoff
.mbar.help.menu add command -label "About" -command aboutCmd -state disabled

frame .map
pack .mbar .map -side top -fill x

proc updateMap {} {
	set statusColor(up) darkGreen
	set statusColor(down) red

	foreach agent [agent cat] {
		.map.$agent configure -background $statusColor([agent status $agent])
	}
	update
}

proc pollMap {} {
	foreach agent [agent cat] {
		chkAgent $agent
	}
	updateMap

	after 5000 pollMap
}

proc mkMap {} {
	foreach agent [agent cat] {
		label .map.$agent -text $agent
		pack .map.$agent -side left -expand 1 -fill x
	}
}

mkMap

foreach agent [agent cat] {
	agent connect $agent
}

pollMap

updateRunButtons
updateAcquisitionButtons
