# the list of sub systems according to your needs, possible sub systems
# are: trig rich0 rich1 rich2 mdc0 tof1 tof2 tof3 shw

set subSystems { trig shw  rich0 rich1 rich2  tof1 tof2 tof3  }
#set subSystems { trig shw rich0 rich1 rich2  }
#set subSystems { trig rich0 rich1 rich2 tof1 tof2 tof3 tof4 }
#set subSystems { trig rich0 rich1 rich2 }

# the component in the HADES_BASE_DIR
set beamTime sep03

set prot UDP

# the name of the event builder
set evtBuilder hadeb02
set evtBuilderIp 140.181.97.116

set watermark [expr 32 * 1024]
set watermark_mdc [expr 16 * 1024]

set bandwidth(0) 00000
set bandwidth(1) 00000
set bandwidth(2) 25000
set bandwidth(3) 25000
set bandwidth(4) 25000
set bandwidth(5) 40000
set bandwidth(6) 40000

set trigCPU "r2f-35"


if {$prot == "UDP"} {
	set addr $evtBuilderIp
	set atmBase 2222
} else {
	set addr 0
	if {$evtBuilder == "hadeb01"} {
		set atmBase 50
	} elseif {$evtBuilder == "hadeb02"} {
		set atmBase 100
	} elseif {$evtBuilder == "hadeb03"} {
		set atmBase 150
	}
}

group create acquisition
group create run
 
set atmOffset 0
set vcc [expr $atmBase + $atmOffset]
agent create $trigCPU hades /hades/usr/hades/$beamTime/trig/hwtrig
worker create $trigCPU:memnet -a -p -1 -w $bandwidth($atmOffset) -o $prot:$addr:$vcc
worker create $trigCPU:readout -s trigconc -a -p -2 -w $watermark -v notice
group add acquisition $trigCPU:memnet
group add acquisition $trigCPU:readout
lappend init $trigCPU:trigconc:init
lappend reset [ list $trigCPU ":" ]
lappend start [ list $trigCPU ":" ]
lappend stop [ list $trigCPU ":" ]
lappend trigReset $trigCPU ". \$HOME/.bash_profile;  echo \"DAQ_SETUP is:\"; echo \$DAQ_SETUP; cd \$HOME/$beamTime/slow; ./trigconc reset"
lappend trigStart $trigCPU ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./trigconc start"
lappend trigStop $trigCPU ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./trigconc stop"
lappend ctustart $trigCPU ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./ctustart"
lappend ctustop $trigCPU ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./ctustop"

lappend inPaths "-i $prot:0:$vcc"

if {[lsearch $subSystems rich0] >= 0} {
	set atmOffset 2
	set vcc [expr $atmBase + $atmOffset]
	agent create r2f-24 hades /hades/usr/hades/$beamTime/rich/hwrich
	worker create r2f-24:memnet -a -p -2 -w $bandwidth($atmOffset) -o $prot:$addr:$vcc
	worker create r2f-24:readout -s rich0 -a -p -1 -w $watermark -v notice
	group add acquisition r2f-24:memnet
	group add acquisition r2f-24:readout
	lappend init r2f-24:rich0:init
	lappend reset [ list r2f-24 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./rich0 reset" ]
	lappend start [ list r2f-24 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./rich0 start" ]
	lappend stop [ list r2f-24 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./rich0 stop" ]
	lappend inPaths "-i $prot:0:$vcc"
}

if {[lsearch $subSystems rich1] >= 0} {
	set atmOffset 3
	set vcc [expr $atmBase + $atmOffset]
	agent create r2f-28 hades /hades/usr/hades/$beamTime/rich/hwrich
	worker create r2f-28:memnet -a -p -2 -w $bandwidth($atmOffset) -o $prot:$addr:$vcc
	worker create r2f-28:readout -s rich1 -a -p -1 -w $watermark -v notice
	group add acquisition r2f-28:memnet
	group add acquisition r2f-28:readout
	lappend init r2f-28:rich1:init
	lappend reset [ list r2f-28 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./rich1 reset" ]
	lappend start [ list r2f-28 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./rich1 start" ]
	lappend stop [ list r2f-28 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./rich1 stop" ]
	lappend inPaths "-i $prot:0:$vcc"
}

if {[lsearch $subSystems rich2] >= 0} {
	set atmOffset 4
	set vcc [expr $atmBase + $atmOffset]
	agent create r2f-27 hades /hades/usr/hades/$beamTime/rich/hwrich
	worker create r2f-27:memnet -a -p -2 -w $bandwidth($atmOffset) -o $prot:$addr:$vcc
	worker create r2f-27:readout -s rich2 -a -p -1 -w $watermark -v notice
	group add acquisition r2f-27:memnet
	group add acquisition r2f-27:readout
	lappend init r2f-27:rich2:init
	lappend reset [ list r2f-27 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./rich2 reset" ]
	lappend start [ list r2f-27 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./rich2 start" ]
	lappend stop [ list r2f-27 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./rich2 stop" ]
	lappend inPaths "-i $prot:0:$vcc"
}

if {[lsearch $subSystems mdc0] >= 0} {
	set atmOffset 5
	set vcc [expr $atmBase + $atmOffset]
	agent create r2f-8  hades /hades/usr/hades/$beamTime/mdc/hwmdc
	worker create r2f-8:memnet -a -p -2 -w $bandwidth($atmOffset) -o $prot:$addr:$vcc
	worker create r2f-8:readout -s mdc0  -a -p -1 -w $watermark_mdc -v notice
	group add acquisition r2f-8:memnet
	group add acquisition r2f-8:readout
	lappend init r2f-8:mdc0:init
	lappend reset [ list r2f-8 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./mdc0 reset" ]
	lappend start [ list r2f-8 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./mdc0 start" ]
	lappend stop [ list r2f-8 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./mdc0 stop" ]
	lappend inPaths "-i $prot:0:$vcc"
}

if {[lsearch $subSystems mdc1] >= 0} {
	set atmOffset 6
	set vcc [expr $atmBase + $atmOffset]
	agent create r2f-32  hades /hades/usr/hades/$beamTime/mdc/hwmdc
	worker create r2f-32:memnet -a -p -2 -w $bandwidth($atmOffset) -o $prot:$addr:$vcc
	worker create r2f-32:readout -s mdc1  -a -p -1 -w $watermark_mdc -v notice
	group add acquisition r2f-32:memnet
	group add acquisition r2f-32:readout
	lappend init r2f-32:mdc1:init
	lappend reset [ list r2f-32 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./mdc1 reset" ]
	lappend start [ list r2f-32 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./mdc1 start" ]
	lappend stop [ list r2f-32 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./mdc1 stop" ]
	lappend inPaths "-i $prot:0:$vcc"
}

if {[lsearch $subSystems shw] >= 0} {
	set atmOffset 1
	set vcc [expr $atmBase + $atmOffset]
	agent create r2-29 hades /hades/usr/hades/$beamTime/shw/hwshw
	worker create r2-29:memnet -a -p -2 -w $bandwidth($atmOffset) -v notice -o $prot:$addr:$vcc
	worker create r2-29:readout -s show -a -p -1 -w $watermark -v notice
	group add acquisition r2-29:memnet
	group add acquisition r2-29:readout
	lappend init r2-29:shw:init
	lappend reset [ list r2-29 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./shw reset" ]
	lappend start [ list r2-29 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./shw start" ]
	lappend stop [ list r2-29 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./shw stop" ]
	lappend inPaths "-i $prot:0:$vcc"
}

if {[lsearch $subSystems tof1] >= 0} {
	agent create e7_25 hades /hades/usr/hades/$beamTime/tof/hwtrig
	lappend init e7_25:tof1:init
	lappend reset [ list e7_25 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof1 reset" ]
	lappend start [ list e7_25 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof1 start" ]
	lappend stop [ list e7_25 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof1 stop" ]
}

if {[lsearch $subSystems tof2] >= 0} {
	agent create e7_21 hades /hades/usr/hades/$beamTime/tof/hwtrig
	lappend init e7_21:tof2:init
	lappend reset [ list e7_21 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof2 reset" ]
	lappend start [ list e7_21 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof2 start" ]
	lappend stop [ list e7_21 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof2 stop" ]
}

if {[lsearch $subSystems tof3] >= 0} {
	agent create e7_41 hades /hades/usr/hades/$beamTime/tof/hwtrig
	lappend init e7_41:tof3:init
	lappend reset [ list e7_41 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof3 reset" ]
	lappend start [ list e7_41 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof3 start" ]
	lappend stop [ list e7_41 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof3 stop" ]
}

if {[lsearch $subSystems tof4] >= 0} {
	agent create e7_28 hades /hades/usr/hades/$beamTime/tof/hwtrig
	lappend init e7_28:tof4:init
	lappend reset [ list e7_28  ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof4 reset" ]
	lappend start [ list e7_28 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof4 start" ]
	lappend stop [ list e7_28 ". \$HOME/.bash_profile;  cd \$HOME/$beamTime/slow; ./tof4 stop" ]
}


agent create $evtBuilder hadaq /home/hadaq/$beamTime/eb/hadaq_tcl
eval worker create $evtBuilder:evtbuild -a -p -2 -m [llength $inPaths] 
eval worker create $evtBuilder:netmem -a -p -1 -m [llength $inPaths] [join $inPaths { }]
group add run $evtBuilder:evtbuild
group add run $evtBuilder:netmem

set duringInit [list "run_parallel_$beamTime [join $init { }]"]

if {1} {
lappend beforeStartAcq "$trigReset"
lappend beforeStartAcq {join ""}
foreach i "$reset" { lappend beforeStartAcq $i }
lappend beforeStartAcq {join ""}
lappend beforeStartAcq "$trigReset"
lappend beforeStartAcq {join ""}
foreach i "$start" { lappend beforeStartAcq $i }
lappend beforeStartAcq {join ""}
lappend beforeStartAcq "$trigStart"
lappend beforeStartAcq {join ""}

lappend afterStartAcq "$ctustart"
lappend afterStartAcq {join ""}

lappend beforeStopAcq "$ctustop"
lappend beforeStopAcq {join ""}

lappend afterStopAcq "$trigStop"
lappend afterStopAcq {join ""}
foreach i "$stop" { lappend afterStopAcq $i }
lappend afterStopAcq {join ""}
} else {
lappend beforeStartAcq {$trigCPU date}
lappend beforeStartAcq {join ""}
lappend afterStartAcq {$trigCPU date}
lappend afterStartAcq {join ""}
lappend beforeStopAcq {$trigCPU date}
lappend beforeStopAcq {join ""}
lappend afterStopAcq {$trigCPU date}
lappend afterStopAcq {join ""}
}










