static char *rcsId = "$Header: /misc/hadaq/cvsroot/hadaq/hadtuqueue.c,v 6.6 2003/05/14 13:54:43 muench Exp $";


#define _ANSI_C_SOURCE
#include <stddef.h>

#include <assert.h>

#include "hadtu.h"

struct HadTuQueueS {
	void *hadTu;
	void *run;
	size_t maxSize;
};

#include "hadtuqueue.h"

int HadTuQueue_invariant(const HadTuQueue *my)
{
	return (
			   my->maxSize > HadTu_hdrSize()
			   && my->run >= HadTu_data(my->hadTu)
			   && my->run <= HadTu_end(my->hadTu)
			   && HadTu_size(my->hadTu) == HadTu_paddedSize(my->hadTu)
	);
}

void conHadTuQueue(HadTuQueue *my, void *mem, size_t maxSize)
{
	my->hadTu = mem;
	HadTu_setSize(my->hadTu, HadTu_hdrSize());
	HadTu_setDecoding(my->hadTu, HadTu_queueDecoding);

	my->run = HadTu_data(my->hadTu);

	/* truncate to aligned size */
	my->maxSize = (maxSize / HadTu_alignment(my->hadTu)) * HadTu_alignment(my->hadTu);

	assert(HadTuQueue_invariant(my));
}

void conHadTuQueue_voidP(HadTuQueue *my, void *mem)
{
	my->hadTu = mem;
	my->run = HadTu_data(my->hadTu);
	my->maxSize = HadTu_size(my->hadTu);

	assert(HadTuQueue_invariant(my));
}

void desHadTuQueue(HadTuQueue *my)
{
	assert(HadTuQueue_invariant(my));
}

size_t HadTuQueue_sizeOf(void)
{
	return sizeof(HadTuQueue);
}

void HadTuQueue_cntl(HadTuQueue *my, int flag)
{
	assert(HadTuQueue_invariant(my));
	assert(flag == HadTuQueueProt_read || flag == HadTuQueueProt_write);

	switch (flag) {
	case HadTuQueueProt_read:
		my->run = HadTu_data(my->hadTu);
		break;
	case HadTuQueueProt_write:
		my->run = HadTu_data(my->hadTu);
		HadTu_setSize(my->hadTu, HadTu_hdrSize());
		break;
	}
	assert(HadTuQueue_invariant(my));
}

void HadTuQueue_clear(HadTuQueue *my)
{
	assert(HadTuQueue_invariant(my));
	my->run = HadTu_data(my->hadTu);
	HadTu_setSize(my->hadTu, HadTu_hdrSize());
	assert(HadTuQueue_invariant(my));
}

int HadTuQueue_empty(const HadTuQueue *my)
{
	assert(HadTuQueue_invariant(my));
	return HadTu_size(my->hadTu) == (char *) my->run - (char *) my->hadTu;
}

size_t HadTuQueue_size(const HadTuQueue *my)
{
	assert(HadTuQueue_invariant(my));
	return HadTu_size(my->hadTu);
}

void *HadTuQueue_alloc(const HadTuQueue *my, size_t size)
{
	void *retVal;

	assert(HadTuQueue_invariant(my));
	assert(size + HadTu_hdrSize() <= my->maxSize);
	if (HadTu_size(my->hadTu) + size > my->maxSize) {
		retVal = NULL;
	} else {
		retVal = my->run;
	}
	return retVal;
}

void HadTuQueue_push(HadTuQueue *my)
{
	assert(HadTuQueue_invariant(my));
	my->run = HadTu_next(my->hadTu, my->run);
	HadTu_setSize(my->hadTu, (char *) my->run - (char *) my->hadTu);
	assert(HadTuQueue_invariant(my));
}

void *HadTuQueue_2voidP(const HadTuQueue *my)
{
	assert(HadTuQueue_invariant(my));
	return HadTu_hdr(my->hadTu);
}

void *HadTuQueue_front(const HadTuQueue *my)
{
	void *retVal;

	assert(HadTuQueue_invariant(my));
	if (HadTuQueue_empty(my)) {
		retVal = NULL;
	} else {
		retVal = my->run;
	}
	return retVal;
}

void HadTuQueue_pop(HadTuQueue *my)
{
	assert(HadTuQueue_invariant(my));
	my->run = HadTu_next(my->hadTu, my->run);
	assert(HadTuQueue_invariant(my));
}
