static char *rcsId = "$Header: /misc/hadaq/cvsroot/hadaq/memnet.c,v 6.18 2004/08/13 10:30:00 hadaq Exp $";

#define _POSIX_C_SOURCE 199309L
#define SYSLOG_NAMES
#include <unistd.h>

#include <stddef.h>

#include <errno.h>
#include <setjmp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include <syslog.h>

#include <allParam.h>

#include "shmtrans.h"
#include "nettrans.h"

#include "worker.h"

typedef struct TheArgsS {
	char outPath[PARAM_MAX_VALUE_LEN];
	unsigned long bandWidth;
	unsigned long isStandalone;
	unsigned long priority;
	unsigned long queueSize;
	char verbosity[PARAM_MAX_VALUE_LEN];
} TheArgs;

static jmp_buf terminateJmp;

void sigHandler(int sig)
{
	longjmp(terminateJmp, sig);
}

static int min(int a, int b)
{
	return a < b ? a : b;
}

static void usage(const char *progName)
{
	syslog(LOG_ERR, "Usage: %s [-a (agentCtrl)] -o outPath", progName);
	syslog(LOG_ERR, "Usage: [-w bandwidth] [-p priority] [-q queueSize]");
}

static void argsDump(TheArgs *my)
{
	syslog(LOG_DEBUG, "outPath: %s", my->outPath);
	syslog(LOG_DEBUG, "bandWidth: %d", my->bandWidth);
	syslog(LOG_DEBUG, "priority: %d", my->priority);
	syslog(LOG_DEBUG, "isStandalone: %d", my->isStandalone);
	syslog(LOG_DEBUG, "queueSize: %d", my->queueSize);
	syslog(LOG_DEBUG, "verbosity: %s", my->verbosity);
}

static void argsDefault(TheArgs *my)
{
	strcpy(my->outPath, "");
	my->bandWidth = 0;
	my->priority = 0;
	my->isStandalone = 1;
	my->queueSize = 1 * 1024 * 1024;
	strcpy(my->verbosity, "info");
}

static int argsFromCL(TheArgs *my, int argc, char *argv[])
{
	extern char *optarg;
	int i;

	while ((i = getopt(argc, argv, "aw:q:o:p:v:")) != -1) {
		switch (i) {
		case 'w':
			my->bandWidth = atoi(optarg);
			break;
		case 'o':
			strcpy(my->outPath, optarg);
			break;
		case 'a':
			my->isStandalone = 0;
			break;
		case 'p':
			my->priority = atoi(optarg);
			break;
		case 'q':
			my->queueSize = atoi(optarg);
			break;
		case 'v':
			strcpy(my->verbosity, optarg);
			break;
		default:
			return -1;
			break;
		}
	}
	return 0;
}

static int argsFromParam(TheArgs *my, int argc, char *argv[])
{
	Param paramS, *param = &paramS;
	int paramWasFound;
	char *name;

	conSetupParam(param, getenv("DAQ_SETUP"));

	name = (char *) basename(argv[0]);
	Param_getString(param, name, "outpath", &paramWasFound, my->outPath);
	Param_getInt(param, name, "bdwidth", &paramWasFound, &my->bandWidth);
	Param_getInt(param, name, "stndln", &paramWasFound, &my->isStandalone);
	Param_getInt(param, name, "prio", &paramWasFound, &my->priority);
	Param_getInt(param, name, "qsize", &paramWasFound, &my->queueSize);
	Param_getString(param, name, "verb", &paramWasFound, my->verbosity);
	desParam(param);
}

static void storeInfoStart(const char *n, time_t t, TheArgs *my)
{
	Param pS, *p = &pS;
	int i;
	char s[20];

	conSetupParam(p, getenv("DAQ_SETUP"));

	strftime(s, 20, "%Y-%m-%dT%H:%M:%S", localtime(&t));
	Param_storeString(p, n, "startdate", s);
	Param_storeString(p, n, "outpath", my->outPath);
	Param_storeInt(p, n, "bdwidth", my->bandWidth);
	Param_storeInt(p, n, "stndln", my->isStandalone);
	Param_storeInt(p, n, "prio", my->priority);
	Param_storeInt(p, n, "qsize", my->queueSize);
	Param_storeString(p, n, "verb", my->verbosity);
	desParam(p);
}

static void storeInfoStop(const char *n, time_t t, Worker *w)
{
	Param pS, *p = &pS;
	int i;
	char s[20];

	conSetupParam(p, getenv("DAQ_SETUP"));

	for (i = 0; i < 32 && strcmp(w->statistics[i].name, "") != 0; i++) {
		Param_storeInt(p, n, w->statistics[i].name, w->statistics[i].value);
	}
	strftime(s, 20, "%Y-%m-%dT%H:%M:%S", localtime(&t));
	Param_storeString(p, n, "stopdate", s);

	desParam(p);
}

int main(int argc, char *argv[])
{
	int i;
	TheArgs theArgsS, *theArgs = &theArgsS;
	Worker *worker;
	NetTrans *netTrans;
	ShmTrans *shmTrans;
	int exitStat;

	openlog(argv[0], LOG_PID | LOG_PERROR, LOG_LOCAL0);
	setlogmask(LOG_UPTO(LOG_INFO));



	argsDefault(theArgs);
	argsFromParam(theArgs, argc, argv);
	if (0 > argsFromCL(theArgs, argc, argv)) {
		usage(argv[0]);
		exit(EXIT_FAILURE);
	}
	for (i = 0; prioritynames[i].c_name != NULL && 0 != strcmp(prioritynames[i].c_name, theArgs->verbosity); i++) {
	}
	if (prioritynames[i].c_name == NULL) {
		exit(EXIT_FAILURE);
	} else {
		setlogmask(LOG_UPTO(prioritynames[i].c_val));
	}

	argsDump(theArgs);

	if (strcmp(theArgs->outPath, "") == 0) {
		usage(argv[0]);
		exit(EXIT_FAILURE);
	}
	if (NULL == (worker = Worker_initBegin(argv[0], sigHandler, theArgs->priority, theArgs->isStandalone))) {
		syslog(LOG_ERR, "initializing working process: %s"), strerror(errno);
		exitStat = EXIT_FAILURE;
		goto bailOut0;
	}
	if (NULL == (shmTrans = ShmTrans_create("subevtqueue", 2 * theArgs->queueSize))) {
		syslog(LOG_ERR, "creating shared memory \"subevtqueue\": %s"), strerror(errno);
		exitStat = EXIT_FAILURE;
		goto bailOut1;
	}
	if (NULL == (netTrans = NetTrans_open(theArgs->outPath, theArgs->bandWidth, worker))) {
		syslog(LOG_ERR, "opening network transport %s: %s", theArgs->outPath, strerror(errno));
		exitStat = EXIT_FAILURE;
		goto bailOut2;
	}
	storeInfoStart(argv[0], time(NULL), theArgs);
	Worker_initEnd(worker);

	while (setjmp(terminateJmp) == 0) {
		void *hadTuQueue;

		Worker_dump(worker, 1);

		hadTuQueue = ShmTrans_recv(shmTrans);
		if (0 > NetTrans_send(netTrans, hadTuQueue)) {
			syslog(LOG_ERR, "sending network packet: %s", strerror(errno));
			exitStat = EXIT_FAILURE;
			goto bailOut3;
		}
		ShmTrans_free(shmTrans);
	}
	exitStat = EXIT_SUCCESS;

	storeInfoStop(argv[0], time(NULL), worker);

  bailOut3:
	NetTrans_close(netTrans);
  bailOut2:
	ShmTrans_remove(shmTrans);
  bailOut1:
	Worker_fini(worker);
  bailOut0:
	exit(exitStat);
}
