static char *rcsId = "$Header: /misc/hadaq/cvsroot/hadaq/netmem.c,v 6.28 2004/08/13 10:30:00 hadaq Exp $";


#define _POSIX_C_SOURCE 199309L
#define SYSLOG_NAMES
#include <unistd.h>

#include <errno.h>
#include <setjmp.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <syslog.h>

#include <allParam.h>

#include "worker.h"
#include "nettrans.h"
#include "shmtrans.h"

#define MAXINPATH 32

typedef struct TheArgsS {
	char inPathS[MAXINPATH][PARAM_MAX_NAME_LEN];
	char *inPath[MAXINPATH];
	int inPathCnt;
	unsigned long nrOfMsgs;
	unsigned long isStandalone;
	unsigned long priority;
	unsigned long queueSize;
	char verbosity[PARAM_MAX_VALUE_LEN];
} TheArgs;

static jmp_buf terminateJmp;

void sigHandler(int sig)
{
	longjmp(terminateJmp, sig);
}

static void usage(const char *progName)
{
	syslog(LOG_ERR, "Usage: %s -i inPath [-i inPath] -m nrOfMsgs [-p priority]", progName);
}

static void argsDump(TheArgs *my)
{
	int i;

	for (i = 0; i < my->inPathCnt; i++) {
		syslog(LOG_DEBUG, "inPath[%d]: %s", i, my->inPath[i]);
	}
	syslog(LOG_DEBUG, "nrOfMsgs: %d", my->nrOfMsgs);
	syslog(LOG_DEBUG, "priority: %d", my->priority);
	syslog(LOG_DEBUG, "isStandalone: %d", my->isStandalone);
	syslog(LOG_DEBUG, "queueSize: %d", my->queueSize);
	syslog(LOG_DEBUG, "verbosity: %s", my->verbosity);
}

static void argsDefault(TheArgs *my)
{
	int i;

	for (i = 0; i < MAXINPATH; i++) {
		my->inPath[i] = my->inPathS[i];
	}
	my->inPathCnt = 0;
	my->nrOfMsgs = 0;
	my->priority = 0;
	my->isStandalone = 1;
	my->queueSize = 1 * 1024 * 1024;
	strcpy(my->verbosity, "info");
}

static int argsFromCL(TheArgs *my, int argc, char *argv[])
{
	extern char *optarg;
	int i;
	int inPathCntFromParam;

	/* inPath must _overwrite_, not add to param */
	inPathCntFromParam = my->inPathCnt;
	my->inPathCnt = 0;

	while ((i = getopt(argc, argv, "ai:m:q:p:v:")) != -1) {
		switch (i) {
		case 'm':
			my->nrOfMsgs = atoi(optarg);
			break;
		case 'i':
			strcpy(my->inPath[my->inPathCnt++], optarg);
			break;
		case 'a':
			my->isStandalone = 0;
			break;
		case 'p':
			my->priority = atoi(optarg);
			break;
		case 'q':
			my->queueSize = atoi(optarg);
			break;
		case 'v':
			strcpy(my->verbosity, optarg);
			break;
		default:
			return -1;
			break;
		}
	}

	/* restore inPathCnt only if it ws not touched here */
	if (my->inPathCnt == 0) {
		my->inPathCnt = inPathCntFromParam;
	}

	return 0;
}

static int argsFromParam(TheArgs *my, int argc, char *argv[])
{
	Param paramS, *param = &paramS;
	int paramWasFound;
	char *name;

	conSetupParam(param, getenv("DAQ_SETUP"));

	name = (char *) basename(argv[0]);
	Param_getStringArray(param, name, "inpath", MAXINPATH, &my->inPathCnt, my->inPath);
	Param_getInt(param, name, "nrofmsgs", &paramWasFound, &my->nrOfMsgs);
	Param_getInt(param, name, "stndln", &paramWasFound, &my->isStandalone);
	Param_getInt(param, name, "prio", &paramWasFound, &my->priority);
	Param_getInt(param, name, "qsize", &paramWasFound, &my->queueSize);
	Param_getString(param, name, "verb", &paramWasFound, my->verbosity);
	desParam(param);
}

static char *unit(unsigned long v)
{
	static char retVal[6];
	static char u[] = " kM";
	int i;

	for (i = 0; v >= 10000 && i < sizeof(u) - 2; v /= 1000, i++) {
	}
	sprintf(retVal, "%4d%c", v, u[i]);

	return retVal;
}

static void statsDump(TheArgs *theArgs, NetTrans **my, int interval)
{
	static unsigned long lastBR[32];
	static time_t lastTime = 0;
	time_t dT;
	int i;


	if (theArgs->isStandalone && strcmp(theArgs->verbosity, "info") == 0 && (dT = (time(NULL) - lastTime)) >= interval) {
		fputs("==============================================================================\n", stderr);
		for (i = 0; i < theArgs->nrOfMsgs; i++) {
			fprintf(stderr, "%17s%02d:%6s", "pktsReceived", i, unit(*my[i]->pktsReceived));
			fprintf(stderr, "%17s%02d:%6s", "pktsDiscarded", i, unit(*my[i]->pktsDiscarded));
			fprintf(stderr, "%15s%02d  :%6s", "bytesReceived", i, unit(*my[i]->bytesReceived));
			fputc('\n', stderr);
			fprintf(stderr, "%17s%02d:%6s", "msgsReceived", i, unit(*my[i]->msgsReceived));
			fprintf(stderr, "%17s%02d:%6s", "msgsDiscarded", i, unit(*my[i]->msgsDiscarded));
			fprintf(stderr, "%15s%02d/s:%6s", "bytesReceived", i, unit(*my[i]->bytesReceived - lastBR[i]));
			lastBR[i] = *my[i]->bytesReceived;
			fputc('\n', stderr);
			fputs("------------------------------------------------------------------------------\n", stderr);
		}
		lastTime += dT;
	}
}

static void storeInfoStart(const char *n, time_t t, TheArgs *my)
{
	Param pS, *p = &pS;
	int i;
	char s[20];

	conSetupParam(p, getenv("DAQ_SETUP"));

	strftime(s, 20, "%Y-%m-%dT%H:%M:%S", localtime(&t));
	Param_storeString(p, n, "startdate", s);
	for (i = 0; i < my->inPathCnt; i++) {
		sprintf(s, "inpath%d", i);
		Param_storeString(p, n, s, my->inPath[i]);
	}
	Param_storeInt(p, n, "nrofmsgs", my->nrOfMsgs);
	Param_storeInt(p, n, "stndln", my->isStandalone);
	Param_storeInt(p, n, "prio", my->priority);
	Param_storeInt(p, n, "qsize", my->queueSize);
	Param_storeString(p, n, "verb", my->verbosity);
	desParam(p);
}

static void storeInfoStop(const char *n, time_t t, Worker *w)
{
	Param pS, *p = &pS;
	int i;
	char s[20];

	conSetupParam(p, getenv("DAQ_SETUP"));

	for (i = 0; i < 32 && strcmp(w->statistics[i].name, "") != 0; i++) {
		Param_storeInt(p, n, w->statistics[i].name, w->statistics[i].value);
	}
	strftime(s, 20, "%Y-%m-%dT%H:%M:%S", localtime(&t));
	Param_storeString(p, n, "stopdate", s);

	desParam(p);
}

int main(int argc, char *argv[])
{
	int i;
	TheArgs theArgsS, *theArgs = &theArgsS;
	Worker *worker;
	NetTrans **netTrans;
	ShmTrans **shmTrans;
	void **hadTu;
	size_t *hadTuSize;

	openlog(argv[0], LOG_PID | LOG_PERROR, LOG_LOCAL0);
	setlogmask(LOG_UPTO(LOG_INFO));



	argsDefault(theArgs);
	argsFromParam(theArgs, argc, argv);
	if (0 > argsFromCL(theArgs, argc, argv)) {
		usage(argv[0]);
		exit(EXIT_FAILURE);
	}
	for (i = 0; prioritynames[i].c_name != NULL && 0 != strcmp(prioritynames[i].c_name, theArgs->verbosity); i++) {
	}
	if (prioritynames[i].c_name == NULL) {
		exit(EXIT_FAILURE);
	} else {
		setlogmask(LOG_UPTO(prioritynames[i].c_val));
	}

	argsDump(theArgs);

	if (theArgs->inPathCnt == 0 || theArgs->nrOfMsgs == 0 || theArgs->inPathCnt != theArgs->nrOfMsgs) {
		usage(argv[0]);
		exit(EXIT_FAILURE);
	}
	netTrans = malloc(theArgs->nrOfMsgs * sizeof(NetTrans *));
	shmTrans = malloc(theArgs->nrOfMsgs * sizeof(ShmTrans *));
	hadTu = malloc(theArgs->nrOfMsgs * sizeof(void *));
	hadTuSize = malloc(theArgs->nrOfMsgs * sizeof(size_t));

	if (NULL == (worker = Worker_initBegin(argv[0], sigHandler, theArgs->priority, theArgs->isStandalone))) {
		syslog(LOG_ERR, "%s, %d: %s", __FILE__, __LINE__, strerror(errno));
		exit(EXIT_FAILURE);
	}
	for (i = 0; i < theArgs->nrOfMsgs; i++) {
		char buf[80];

		netTrans[i] = NetTrans_create(theArgs->inPath[i], 0, worker);
		if (netTrans[i] == NULL) {
			syslog(LOG_ERR, "%s, %d: %s", __FILE__, __LINE__, strerror(errno));
			exit(EXIT_FAILURE);
		}
		sprintf(buf, "netqueue%d", i);
		shmTrans[i] = ShmTrans_open(buf, 2 * theArgs->queueSize);
		if (shmTrans[i] == NULL) {
			syslog(LOG_ERR, "%s, %d: %s", __FILE__, __LINE__, strerror(errno));
			exit(EXIT_FAILURE);
		}
		hadTu[i] = NULL;
		hadTuSize[i] = theArgs->queueSize - HadTu_hdrSize();
	}
	storeInfoStart(argv[0], time(NULL), theArgs);
	Worker_initEnd(worker);

	if (setjmp(terminateJmp) == 0) {
		while (1) {
			long msgsCompleted;

			statsDump(theArgs, netTrans, 1);

			for (i = 0; i < theArgs->nrOfMsgs; i++) {
				if (hadTu[i] == NULL) {
					ShmTrans_requestSpace(shmTrans[i]);
					hadTu[i] = ShmTrans_tryAlloc(shmTrans[i], hadTuSize[i]);
				}
			}

			msgsCompleted = NetTrans_multiRecv(netTrans, hadTu, hadTuSize, theArgs->nrOfMsgs);

			for (i = 0; i < theArgs->nrOfMsgs; i++) {
				if ((msgsCompleted & (1 << i)) != 0) {
					ShmTrans_send(shmTrans[i]);
					hadTu[i] = NULL;
				}
			}
		}
	}
	storeInfoStop(argv[0], time(NULL), worker);
	statsDump(theArgs, netTrans, 0);

	for (i = 0; i < theArgs->nrOfMsgs; i++) {
		ShmTrans_remove(shmTrans[i]);
		NetTrans_remove(netTrans[i]);
	}
	Worker_fini(worker);
	exit(EXIT_SUCCESS);
}
