
#ifndef NETTRANS_H
#define NETTRANS_H

#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>

#include "worker.h"
#include "hadtu.h"

typedef enum NetTransTypeE {
	NetTransType_invalid,
	NetTransType_udp
} NetTransType;

typedef struct NetTransS {
	char name[512];
	NetTransType type;
	union {
		struct sockaddr_in sa;
	} spec;
	size_t mtuSize;
	int fd;
	void *pkt;
	size_t pktSize;
	size_t offset;
	unsigned long *pktsSent;
	unsigned long *msgsSent;
	unsigned long *bytesSent;
	unsigned long *pktsReceived;
	unsigned long *pktsDiscarded;
	unsigned long *msgsReceived;
	unsigned long *msgsDiscarded;
	unsigned long *bytesReceived;
} NetTrans;

int NetTrans_invariant(const NetTrans *my);
NetTrans *NetTrans_create(const char *name, size_t bandwidth, Worker *worker);
NetTrans *NetTrans_open(const char *name, size_t bandwidth, Worker *worker);
void NetTrans_close(NetTrans *my);
void NetTrans_remove(NetTrans *my);

int NetTrans_send(NetTrans *my, void *hadTu);
void NetTrans_recv(NetTrans *my, void *hadTu, size_t size);
unsigned long NetTrans_multiRecv(NetTrans *my[], void *hadTu[], size_t size[], int nrOfMsgs);

#endif
