
static char *rcsId = "$Header: /misc/hadaq/cvsroot/hadaq/readout.c,v 6.29 2004/08/13 10:30:00 hadaq Exp $";

#define _POSIX_C_SOURCE 199309L
#define SYSLOG_NAMES
#include <unistd.h>
#include "grmblfx.h"

#include <errno.h>
#include <stdlib.h>
#include <setjmp.h>
#include <string.h>
#include <sys/types.h>

#include <syslog.h>

#include <allParam.h>

#include "hardware.h"
#include "shmtrans.h"
#include "hadtuqueue.h"
#include "worker.h"
#include "subevt.h"

typedef struct TheArgsS {
	unsigned long waterMark;
	unsigned long isStandalone;
	unsigned long priority;
	unsigned long queueSize;
	char verbosity[PARAM_MAX_VALUE_LEN];
	char subsystem[PARAM_MAX_VALUE_LEN];
} TheArgs;

static jmp_buf terminateJmp;

void sigHandler(int sig)
{
	longjmp(terminateJmp, sig);
}

static void usage(const char *progName)
{
	syslog(LOG_ERR, "Usage: %s [-a (agentCtrl)] [-p priority]", progName);
	syslog(LOG_ERR, "Usage: [-q queueSize] [-w waterMark]");
}

static void argsDump(TheArgs *my)
{
	syslog(LOG_DEBUG, "waterMark: %d", my->waterMark);
	syslog(LOG_DEBUG, "priority: %d", my->priority);
	syslog(LOG_DEBUG, "isStandalone: %d", my->isStandalone);
	syslog(LOG_DEBUG, "queueSize: %d", my->queueSize);
	syslog(LOG_DEBUG, "verbosity: %s", my->verbosity);
	syslog(LOG_DEBUG, "subsystem: %s", my->subsystem);
}

static void argsDefault(TheArgs *my)
{
	my->waterMark = 56 * 1024;
	my->priority = 0;
	my->isStandalone = 1;
	my->queueSize = 1 * 1024 * 1024;
	strcpy(my->verbosity, "info");
	strcpy(my->subsystem, "unknown");
}

static int argsFromCL(TheArgs *my, int argc, char *argv[])
{
	extern char *optarg;
	int i;

	while ((i = getopt(argc, argv, "w:ap:q:s:v:")) != -1) {
		switch (i) {
		case 'w':
			my->waterMark = atoi(optarg);
			break;
		case 'a':
			my->isStandalone = 0;
			break;
		case 'p':
			my->priority = atoi(optarg);
			break;
		case 'q':
			my->queueSize = atoi(optarg);
			break;
		case 's':
			strcpy(my->subsystem, optarg);
			break;
		case 'v':
			strcpy(my->verbosity, optarg);
			break;
		default:
			return -1;
			break;
		}
	}
	return 0;
}

static int argsFromParam(TheArgs *my, int argc, char *argv[])
{
	Param paramS, *param = &paramS;
	int paramWasFound;
	char *name;
	conSetupParam(param, getenv("DAQ_SETUP"));
	name = (char *) basename(argv[0]);
	Param_getInt(param, name, "wmark", &paramWasFound, &my->waterMark);
	Param_getInt(param, name, "stndln", &paramWasFound, &my->isStandalone);
	Param_getInt(param, name, "prio", &paramWasFound, &my->priority);
	Param_getInt(param, name, "qsize", &paramWasFound, &my->queueSize);
	Param_getString(param, name, "verb", &paramWasFound, my->verbosity);
	Param_getString(param, name, "subsystem", &paramWasFound, my->subsystem);
	desParam(param);
}

static void storeInfoStart(const char *n, time_t t, TheArgs *my)
{
	Param pS, *p = &pS;
	int i;
	char s[20];
	conSetupParam(p, getenv("DAQ_SETUP"));
	strftime(s, 20, "%Y-%m-%dT%H:%M:%S", localtime(&t));
	Param_storeString(p, n, "startdate", s);
	Param_storeInt(p, n, "wmark", my->waterMark);
	Param_storeInt(p, n, "stndln", my->isStandalone);
	Param_storeInt(p, n, "prio", my->priority);
	Param_storeInt(p, n, "qsize", my->queueSize);
	Param_storeString(p, n, "verb", my->verbosity);
	Param_storeString(p, n, "subsystem", my->subsystem);
	desParam(p);
}

static void storeInfoStop(const char *n, time_t t, Worker *w)
{
	Param pS, *p = &pS;
	int i;
	char s[20];
	conSetupParam(p, getenv("DAQ_SETUP"));
	for (i = 0; i < 32 && strcmp(w->statistics[i].name, "") != 0; i++) {
		Param_storeInt(p, n, w->statistics[i].name, w->statistics[i].value);
	}
	strftime(s, 20, "%Y-%m-%dT%H:%M:%S", localtime(&t));
	Param_storeString(p, n, "stopdate", s);

	desParam(p);
}

int readoutMain(int argc, char *argv[])
{
	int i;
	TheArgs theArgsS, *theArgs = &theArgsS;
	Hardware *hw;
	void *subEvt;
//printf("*subEvt_1 = 0x%x\n", subEvt);
	Worker *worker;
	ShmTrans *shmTrans;
	unsigned long *trigAccepted;
	unsigned long *subevtsRead;
	unsigned long *subevtsDataErr;

	openlog(argv[0], LOG_PID | LOG_PERROR, LOG_LOCAL0);
	setlogmask(LOG_UPTO(LOG_INFO));

	argsDefault(theArgs);
	argsFromParam(theArgs, argc, argv);
	if (0 > argsFromCL(theArgs, argc, argv)) {
		usage(argv[0]);
		goto bailOut0;
	}
	for (i = 0; prioritynames[i].c_name != NULL && 0 != strcmp(prioritynames[i].c_name, theArgs->verbosity); i++) {
	}
	if (prioritynames[i].c_name == NULL) {
		goto bailOut0;
	} else {
		setlogmask(LOG_UPTO(prioritynames[i].c_val));
	}

	argsDump(theArgs);
	if (NULL == (worker = Worker_initBegin(argv[0], sigHandler, theArgs->priority, theArgs->isStandalone))) {
		syslog(LOG_ERR, "connecting to agent: %s", strerror(errno));
		goto bailOut0;
	}
	if (NULL == (hw = newHardware(theArgs->subsystem))) {
		syslog(LOG_ERR, "initializing hardware: %s", strerror(errno));
		goto bailOut1;
	}
	if (NULL == (shmTrans = ShmTrans_open("subevtqueue", 2 * theArgs->queueSize))) {
		syslog(LOG_ERR, "opening shared memory \"%s\": %s", "subevtqueue", strerror(errno));
		goto bailOut2;
	}
	trigAccepted = Worker_addStatistic(worker, "trigAccepted");
	subevtsRead = Worker_addStatistic(worker, "subevtsRead");
	subevtsDataErr = Worker_addStatistic(worker, "subevtsDataErr");

	storeInfoStart(argv[0], time(NULL), theArgs);
	Worker_initEnd(worker);


	/* at somewhat more than the HadTuQueue header must fit into watermark */
	if (theArgs->waterMark < HadTu_hdrSize()) {
		theArgs->waterMark = HadTu_hdrSize();
	}
	while (setjmp(terminateJmp) == 0) {
		void *hadTu;
//printf("*hadTu_1 = 0x%x\n", hadTu);
		HadTuQueue *hadTuQueue;
//printf("*hadTuQueue_1 = 0x%x\n", hadTuQueue);
		size_t transportSize = theArgs->waterMark + Hardware_maxSubEvtSize(hw) + +sizeof(long);	/* BUGBUG HadTuQueue truncates size to alignment, so add sizeof(long) here */


		Worker_dump(worker, 1);

		hadTu = ShmTrans_alloc(shmTrans, transportSize);
//printf("*hadTu_2 = 0x%x\n", hadTu);
#ifndef NDEBUG
		syslog(LOG_DEBUG, "shmTrans: %p = hadTu: %s", hadTu, HadTu_2charP(hadTu));
#endif
		hadTuQueue = malloc(HadTuQueue_sizeOf());
//printf("*hadTuQueue_2 = 0x%x\n", hadTuQueue);
		conHadTuQueue(hadTuQueue, hadTu, transportSize);
//printf("*hadTuQueue_3 = 0x%x\n", hadTuQueue);
//printf("*hadTu_3 = 0x%x\n", hadTu);
		while (HadTuQueue_size(hadTuQueue) <= theArgs->waterMark
			   && NULL != (subEvt = HadTuQueue_alloc(hadTuQueue, Hardware_maxSubEvtSize(hw)))
			) {
			Hardware_waitForTrigger(hw, subEvt);/*nothing with subEvt*/
//printf("*subEvt_2 = 0x%x\n", subEvt);
			Hardware_readout(hw, subEvt);/*whole event into subEvt*/
//printf("*subEvt_3 = 0x%x\n", subEvt);
			(*subevtsRead)++;
			if (SubEvt_dataError(subEvt)) {
				(*subevtsDataErr)++;
			}
			(*trigAccepted) = SubEvt_trigNr(subEvt) >> 8;
//printf("*subEvt_4 = 0x%x\n", subEvt);
//printf("*hadTuQueue_4 = 0x%x\n", hadTuQueue);
#ifndef NDEBUG
			syslog(LOG_DEBUG, "hadTuQueue: %p = subEvt: %s", subEvt, SubEvt_2charP(subEvt));
#endif

			HadTuQueue_push(hadTuQueue);
//printf("*hadTuQueue_5 = 0x%x\n", hadTuQueue);
		}
		desHadTuQueue(hadTuQueue);
		free(hadTuQueue);
		ShmTrans_send(shmTrans);
	}
	storeInfoStop(argv[0], time(NULL), worker);

	Worker_fini(worker);
	ShmTrans_close(shmTrans);
	deleteHardware(hw);

	exit(EXIT_SUCCESS);

	ShmTrans_close(shmTrans);
  bailOut2:
	deleteHardware(hw);
  bailOut1:
	Worker_fini(worker);
  bailOut0:
	exit(EXIT_FAILURE);
}
