static char *rcsId = "$Header: /misc/hadaq/cvsroot/hadaq/shmtrans.c,v 6.9 2003/05/14 13:54:45 muench Exp $";


#define _POSIX_C_SOURCE 199309L
#include <unistd.h>
#ifndef _POSIX_SEMAPHORES
#error POSIX_SEMAPHORES not available on this OS
#endif

#include <stdlib.h>
#include "grmblfx.h"

#include <errno.h>
#include <fcntl.h>
#include <limits.h>
//#include <semaphore.h>
#include <string.h>
#include <sys/stat.h>

#include <syslog.h>


#include "psxshm.h"
#include "hadtuqueue.h"

#include "shmtrans.h"

#define SPACE_IN_FRONT 16		/* Reserve a little space in front of the
                                   hadtuqueues that is not used, but
                                   accessible. Allows for in situ asembly of
                                   fragments with headers. */

static void switchStorage(ShmTrans *my)
{
	HadTuQueue *queue;

	queue = my->rdQueue;
	my->rdQueue = my->wrQueue;
	my->wrQueue = queue;
}

ShmTrans *ShmTrans_create(const char *name, size_t size)
{
	ShmTrans *my;
	char ipcName[_POSIX_PATH_MAX];
	unsigned long pageSize;
	size_t shmSize;
	char *addr;
	my = malloc(sizeof(ShmTrans));
	my->wrQueue = (HadTuQueue *) malloc(HadTuQueue_sizeOf());
	my->rdQueue = (HadTuQueue *) malloc(HadTuQueue_sizeOf());
	strcpy(my->name, name);

	strcpy(ipcName, my->name);
	strcat(ipcName, ".shm");
	if (0 > PsxShm_unlink(ipcName)) {
		switch (errno) {
		case ENOENT:
			break;
		default:
			syslog(LOG_DEBUG,
				   "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
			goto bailOut1;
			break;
		}
	}
	if (0 > (pageSize = sysconf(_SC_PAGESIZE))) {
		syslog(LOG_DEBUG, "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
		goto bailOut1;
	}
	shmSize = 2 * (size + SPACE_IN_FRONT);
	shmSize = (shmSize & (pageSize - 1))
		? (shmSize & ~(pageSize - 1)) + pageSize : shmSize;

	if (NULL == (my->shm = PsxShm_open(ipcName, O_CREAT | O_RDWR, S_IRWXU, shmSize))) {
		syslog(LOG_DEBUG, "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
		goto bailOut1;
	}
	addr = my->shm->addr;
	addr += SPACE_IN_FRONT;
	conHadTuQueue(my->wrQueue, addr, size);
	addr += size;
	addr += SPACE_IN_FRONT;
	conHadTuQueue(my->rdQueue, addr, size);
	HadTuQueue_cntl(my->wrQueue, HadTuQueueProt_write);
	HadTuQueue_cntl(my->rdQueue, HadTuQueueProt_write);

	strcpy(ipcName, my->name);
	strcat(ipcName, "SR.sem");
	if (0 > sem_unlink(ipcName)) {
		switch (errno) {
		case ENOENT:
			break;
		default:

			syslog(LOG_DEBUG,
				   "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
			goto bailOut2;
			break;
		}
	}
	if ((sem_t *) -1 == (my->switchRequest = sem_open(ipcName, O_CREAT | O_EXCL, S_IRWXU, 0))) {
		syslog(LOG_DEBUG, "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
		goto bailOut2;
	}
	strcpy(ipcName, my->name);
	strcat(ipcName, "SC.sem");
	if (0 > sem_unlink(ipcName)) {
		switch (errno) {
		case ENOENT:
			break;
		default:
			syslog(LOG_DEBUG,
				   "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
			goto bailOut3;
			break;
		}
	}
	if ((sem_t *) -1 == (my->switchComplete = sem_open(ipcName, O_CREAT | O_EXCL, S_IRWXU, 0))) {
		syslog(LOG_DEBUG, "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
		goto bailOut3;
	}
	return my;

  bailOut3:
	sem_close(my->switchRequest);
	strcpy(ipcName, my->name);
	strcat(ipcName, "SC.sem");
	sem_unlink(ipcName);
  bailOut2:
	desHadTuQueue(my->rdQueue);
	desHadTuQueue(my->wrQueue);

	PsxShm_close(my->shm);
	strcpy(ipcName, my->name);
	strcat(ipcName, ".shm");
	PsxShm_unlink(ipcName);
  bailOut1:
	free(my->wrQueue);
	free(my->rdQueue);
	free(my);
	return NULL;
}

void ShmTrans_remove(ShmTrans *my)
{
	char ipcName[_POSIX_PATH_MAX];

	sem_close(my->switchRequest);
	strcpy(ipcName, my->name);
	strcat(ipcName, "SR.sem");
	sem_unlink(ipcName);

	sem_close(my->switchComplete);
	strcpy(ipcName, my->name);
	strcat(ipcName, "SC.sem");
	sem_unlink(ipcName);

	desHadTuQueue(my->rdQueue);
	desHadTuQueue(my->wrQueue);

	PsxShm_close(my->shm);
	strcpy(ipcName, my->name);
	strcat(ipcName, ".shm");
	PsxShm_unlink(ipcName);

	free(my->wrQueue);
	free(my->rdQueue);
	free(my);
}

ShmTrans *ShmTrans_open(const char *name, size_t size)
{
	ShmTrans *my;
	char ipcName[_POSIX_PATH_MAX];
	unsigned long pageSize;
	size_t shmSize;
	char *addr;

	my = malloc(sizeof(ShmTrans));

	my->wrQueue = (HadTuQueue *) malloc(HadTuQueue_sizeOf());
	my->rdQueue = (HadTuQueue *) malloc(HadTuQueue_sizeOf());

	strcpy(my->name, name);

	strcpy(ipcName, my->name);
	strcat(ipcName, ".shm");

	if (0 > (pageSize = sysconf(_SC_PAGESIZE))) {
		syslog(LOG_DEBUG, "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
		goto bailOut1;
	}
	shmSize = 2 * (size + SPACE_IN_FRONT);
	shmSize = (shmSize & (pageSize - 1))
		? (shmSize & ~(pageSize - 1)) + pageSize : shmSize;

	if (NULL == (my->shm = PsxShm_open(ipcName, O_RDWR, 0, shmSize))) {
		syslog(LOG_DEBUG, "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
		goto bailOut1;
	}
	addr = my->shm->addr;
	addr += SPACE_IN_FRONT;
	conHadTuQueue(my->wrQueue, addr, size);
	addr += size;
	addr += SPACE_IN_FRONT;
	conHadTuQueue(my->rdQueue, addr, size);
	HadTuQueue_cntl(my->wrQueue, HadTuQueueProt_read);
	HadTuQueue_cntl(my->rdQueue, HadTuQueueProt_read);

	strcpy(ipcName, my->name);
	strcat(ipcName, "SR.sem");

	if ((sem_t *) -1 == (my->switchRequest = sem_open(ipcName, 0, 0, 0))) {
		syslog(LOG_DEBUG, "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
		goto bailOut2;
	}
	strcpy(ipcName, my->name);
	strcat(ipcName, "SC.sem");

	if ((sem_t *) -1 == (my->switchComplete = sem_open(ipcName, 0, 0, 0))) {
		syslog(LOG_DEBUG, "%s, line %d: %s", __FILE__, __LINE__, strerror(errno));
		goto bailOut3;
	}
	return my;

  bailOut3:
	sem_close(my->switchRequest);
  bailOut2:
	desHadTuQueue(my->rdQueue);
	desHadTuQueue(my->wrQueue);

	PsxShm_close(my->shm);
  bailOut1:
	free(my->wrQueue);
	free(my->rdQueue);
	free(my);

	return NULL;
}

void ShmTrans_close(ShmTrans *my)
{
	sem_close(my->switchComplete);
	sem_close(my->switchRequest);
	desHadTuQueue(my->rdQueue);
	desHadTuQueue(my->wrQueue);
	PsxShm_close(my->shm);
	free(my->wrQueue);
	free(my->rdQueue);
	free(my);
}

void *ShmTrans_alloc(ShmTrans *my, size_t size)
{
	void *storage;

	if (sem_trywait(my->switchRequest) == 0) {
		HadTuQueue_cntl(my->wrQueue, HadTuQueueProt_read);
		switchStorage(my);
		HadTuQueue_cntl(my->wrQueue, HadTuQueueProt_write);
		sem_post(my->switchComplete);
	}
	while (NULL == (storage = HadTuQueue_alloc(my->wrQueue, size))) {
		HadTuQueue_cntl(my->wrQueue, HadTuQueueProt_read);
		sem_wait(my->switchRequest);
		switchStorage(my);
		sem_post(my->switchComplete);
		HadTuQueue_cntl(my->wrQueue, HadTuQueueProt_write);
	}
	return storage;
}

void *ShmTrans_tryAlloc(ShmTrans *my, size_t size)
{
	return HadTuQueue_alloc(my->wrQueue, size);
}

void ShmTrans_requestSpace(ShmTrans *my)
{
	if (sem_trywait(my->switchRequest) == 0) {
		HadTuQueue_cntl(my->wrQueue, HadTuQueueProt_read);
		switchStorage(my);
		HadTuQueue_cntl(my->wrQueue, HadTuQueueProt_write);
		sem_post(my->switchComplete);
	}
}

void ShmTrans_send(ShmTrans *my)
{
	HadTuQueue_push(my->wrQueue);
}

void *ShmTrans_recv(ShmTrans *my)
{
	void *storage;

	while (NULL == (storage = HadTuQueue_front(my->rdQueue))) {
		HadTuQueue_cntl(my->rdQueue, HadTuQueueProt_write);
		sem_post(my->switchRequest);
		sem_wait(my->switchComplete);
		switchStorage(my);
		HadTuQueue_cntl(my->rdQueue, HadTuQueueProt_read);
	}
	return storage;
}

void *ShmTrans_recvWhole(ShmTrans *my)
{
	while (HadTuQueue_empty(my->rdQueue)) {
		HadTuQueue_cntl(my->rdQueue, HadTuQueueProt_write);
		sem_post(my->switchRequest);
		sem_wait(my->switchComplete);
		switchStorage(my);
		HadTuQueue_cntl(my->rdQueue, HadTuQueueProt_read);
	}
	return HadTuQueue_2voidP(my->rdQueue);
}

void *ShmTrans_tryRecv(ShmTrans *my)
{
	return HadTuQueue_front(my->rdQueue);
}

void ShmTrans_requestData(ShmTrans *my)
{
	HadTuQueue_cntl(my->rdQueue, HadTuQueueProt_write);
	sem_post(my->switchRequest);
	sem_wait(my->switchComplete);
	switchStorage(my);
	HadTuQueue_cntl(my->rdQueue, HadTuQueueProt_read);
}

void ShmTrans_free(ShmTrans *my)
{
	HadTuQueue_pop(my->rdQueue);
}

void ShmTrans_freeWhole(ShmTrans *my)
{
	HadTuQueue_clear(my->rdQueue);
}
