static char *rcsId = "$Header: /misc/hadaq/cvsroot/hadaq/tcldaq.c,v 6.14 2003/05/14 13:54:45 muench Exp $";

#define _GNU_SOURCE
#include <unistd.h>

#include <stddef.h>

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include <rpc/rpc.h>

#include <tcl.h>
#include "worker.h"
#include "agent.h"

#define MAX_NTHR 16
static nThr = 0;
static char agents[MAX_NTHR][512];

typedef struct ThrArgsS {
	int id;
	char *agent;
	char *command;
}

ThrArgs;

int AgentCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
	CLIENT *cl;

	if (argc != 3) {
		Tcl_AppendResult(interp, "usage: c_agent connect|disconnect|ping <agent>", NULL);
		return TCL_ERROR;
	}
	if (strcmp(argv[1], "connect") == 0) {
		if (NULL == (cl = clnt_create(argv[2], DAQAGENTPROG, DAQAGENTVERS, "tcp"))) {
			Tcl_AppendResult(interp, clnt_spcreateerror(argv[2]), NULL);
			return TCL_ERROR;
		}
		sprintf(interp->result, "%p", cl);
	} else if (strcmp(argv[1], "disconnect") == 0) {
		sscanf(argv[2], "%p", &cl);
		clnt_destroy(cl);
	} else if (strcmp(argv[1], "ping") == 0) {
	} else {
		Tcl_AppendResult(interp, "usage: c_agent connect|disconnect <agent>", NULL);
		return TCL_ERROR;
	}
	return TCL_OK;
}

int WorkerCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
	CLIENT *cl;
	int id, *idP;

	if (argc < 4) {
		Tcl_AppendResult(interp, "usage: c_worker start|stop|status <agent> <worker> [worker options]", NULL);
		return TCL_ERROR;
	}
	sscanf(argv[2], "%p", &cl);
	if (strcmp(argv[1], "start") == 0) {
		RpcWorker_startArgs startArgsS, *startArgs = &startArgsS;

		startArgs->name = argv[3];
		/* CAUTION This does not pass the closing NULL pointer in argv */
		startArgs->argv.argv_len = argc - 3;
		startArgs->argv.argv_val = argv + 3;

		if (NULL == (idP = rpcworker_start_1(startArgs, cl))) {
			Tcl_AppendResult(interp, clnt_sperror(cl, "rpcworker_start_1"), NULL);
			return TCL_ERROR;
		}
		id = *idP;
		sprintf(interp->result, "%d", id);
	} else if (strcmp(argv[1], "stop") == 0) {
		int *res;

		idP = &id;
		sscanf(argv[3], "%d", idP);
		if (NULL == (res = rpcworker_stop_1(idP, cl))) {
			Tcl_AppendResult(interp, clnt_sperror(cl, "rpcworker_stop_1"), NULL);
			return TCL_ERROR;
		}
	} else if (strcmp(argv[1], "status") == 0) {
		char *sS, **s = &sS;

		idP = &id;
		sscanf(argv[3], "%d", idP);
		if (NULL == (s = rpcworker_status_1(idP, cl))) {
			Tcl_AppendResult(interp, clnt_sperror(cl, "rpcworker_status_1"), NULL);
			return TCL_ERROR;
		}
		Tcl_AppendResult(interp, *s, NULL);
	} else {
		Tcl_AppendResult(interp, "usage: c_worker start|stop|status <worker> [options]", NULL);
		return TCL_ERROR;
	}

	return TCL_OK;
}

int SystemCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
	int SystemCmdR;
	CLIENT *cl;

	if (argc < 3) {
		Tcl_AppendResult(interp, "usage: c_system <agent> command", NULL);
		return TCL_ERROR;
	}
	strcpy(agents[nThr++], argv[1]);
	if (NULL == (cl = clnt_create(argv[1], DAQAGENTPROG, DAQAGENTVERS, "tcp"))) {
		Tcl_AppendResult(interp, clnt_spcreateerror(argv[1]), NULL);
		SystemCmdR = TCL_ERROR;
	} else {
		struct timeval TIMEOUT =
		{0, 0};
		struct rpc_err rpcErr;

		clnt_control(cl, CLSET_TIMEOUT, (char *) &TIMEOUT);
		rpcsystem_1(&argv[2], cl);	/* will produce a timeout */
		clnt_geterr(cl, &rpcErr);

		if (rpcErr.re_status != RPC_TIMEDOUT) {	/* a real error */
			Tcl_AppendResult(interp, clnt_sperror(cl, "rpcsystem_1"), NULL);
			SystemCmdR = TCL_ERROR;
		} else {
			SystemCmdR = TCL_OK;
		}
		clnt_destroy(cl);
	}

	return SystemCmdR;
}

int JoinAllCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
	int JoinAllCmdR;
	CLIENT *cl;
	int dummy;

	JoinAllCmdR = TCL_OK;
	while (nThr > 0) {
		--nThr;
		if (NULL == (cl = clnt_create(agents[nThr], DAQAGENTPROG, DAQAGENTVERS, "tcp"))) {
			Tcl_AppendResult(interp, clnt_spcreateerror(argv[1]), NULL);
			JoinAllCmdR = TCL_ERROR;
		} else {
			struct timeval TIMEOUT =
			{120, 0};

			clnt_control(cl, CLSET_TIMEOUT, (char *) &TIMEOUT);
			if (NULL == rpcjoin_1(&dummy, cl)) {
				Tcl_AppendResult(interp, clnt_sperror(cl, "rpcjoin_1"), NULL);
				JoinAllCmdR = TCL_ERROR;
			}
			clnt_destroy(cl);
		}
	}

	return JoinAllCmdR;
}

int Daq_Init(Tcl_Interp * interp)
{
	/*
   * Call Tcl_CreateCommand for application-specific commands, if
   * they weren't already created by the init procedures called above.
   */
	Tcl_CreateCommand(interp, "c_worker", WorkerCmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "c_agent", AgentCmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "c_system", SystemCmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);
	Tcl_CreateCommand(interp, "c_joinall", JoinAllCmd, (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

	/*
   * Specify a user-specific startup file to invoke if the application
   * is run interactively.  Typically the startup file is "~/.apprc"
   * where "app" is the name of the application.  If this line is deleted
   * then no user-specific startup file will be run under any conditions.
   */
	return TCL_OK;
}
